;
; Correlation detection test. Bootstrap is performed for both variance and confidence interval estimation.
;
; AUTHOR: Francois-Xavier Dupe
; DATE: 27/08/10
; REVISION: 
;          

;
; waveletCorr: Do the correlation detection inside the wavelet domain using mexican wavelet on given scales
; INPUT:  icmbobs:     the observed CMB
;         igalobs:     the LSS surveys overdensity
;         scales:      the different involved scales
;         nested:      set to true if the maps are in nested format
;         boostrap:    number of bootstrap iterations for the confidence intervals
;         btstpvar:    number of bootstrap iterations for the variance estimation
;         nosmooth:    do not use kernel smoothing for p-value estimation
;         noverb:      disable text printing
; OUTPUT: the score with the correlation and its variance
;
function waveletCorr,icmbobs,igalobs,scales=scales,mask=mask,nested=nested,bootstrap=bootstrap,btstpvar=btstpvar,nosmooth=nosmooth,noverb=noverb,$
                     cmbspec=cmbspec,lmin=lmin,lmax=lmax

  npix   = n_elements(icmbobs)
  nside  = npix2nside(npix)
  nscale = n_elements(scales)
  cmbobs = icmbobs
  galobs = igalobs

  ;; 1 - Convert data to ring convention if needed

  mrs_almtrans, icmbobs, icmbalm, /tab, /complex, lmax=lmax
  mrs_almtrans, igalobs, igalalm, /tab, /complex, lmax=lmax

  almcount = 0ul
  for l=lmin,lmax do almcount = almcount+2*l+1ul

  ;; Remove some poles for coherence with input data
  if lmin gt 0 then begin
     icmbalm.alm[0:lmin-1,*] = dcomplex(0.0d,0.0d)
     mrs_almrec, icmbalm, icmbobs 
     
     igalalm.alm[0:lmin-1,*] = dcomplex(0.0d,0.0d)
     mrs_almrec, igalalm, igalobs
  endif

  ;; 1bis - Do some estimations from perturbed observation
  ;; Use the theoritical spectra or estimate them

  cnn = dblarr(lmax+1)
  for l=0,lmax do begin
     vcnn = dblarr(2*l+1)
     for m=0,l do begin 
        vcnn[m]   = abs(icmbalm.alm[l,m])^2
        vcnn[l+m] = vcnn[m]     ; The m=0 value will be overwritten by m=l
     endfor
     cnn[l] = mean(vcnn)
  endfor

  if keyword_set(cmbspec) then cnn = cmbspec

  if keyword_set(nested) then begin
     ud_grade,icmbobs,cmbobs,order_in='NESTED',order_out='RING'
     ud_grade,igalobs,galobs,order_in='NESTED',order_out='RING'
  endif

  ;; 2 - Compute wavelets coefficient

  wcmb = dblarr(nscale*npix)
  wgal = dblarr(nscale*npix)
  
  varcmb = dblarr(nscale)
  vargal = dblarr(nscale)

  for sc = 0,nscale-1 do begin
     wcmb[sc*npix:(sc+1)*npix-1] = mrs_wtmexhat(cmbobs,scales[sc])
     varcmb[sc] = variance(wcmb[sc*npix:(sc+1)*npix-1])

     wgal[sc*npix:(sc+1)*npix-1] = mrs_wtmexhat(galobs,scales[sc])
     vargal[sc] = variance(wgal[sc*npix:(sc+1)*npix-1])
  endfor

  ;; 3 - Compute the correlation coefficient

  corrCoeff = ( transpose(wcmb) # wgal )
  corrCoeff = corrCoeff[0]

  normGal   = ( transpose(wgal) # wgal )
  normGal   = normGal[0]

  corrCoeff = corrCoeff / normGal

  ;; 4 - Bootstrap for variance estimation (using parametric bootstrap)
  if btstpvar gt 0 then begin

     bootvar = dblarr(btstpvar)    
     tmpalm = icmbalm
     newcmb = dblarr(nscale*npix)

     for iter=0,btstpvar-1 do begin
        
        cmbsignal = dcomplexarr(lmax+1,lmax+1)
        for l=lmin,lmax do begin
           rvar = dblarr(lmax+1)
           ivar = dblarr(lmax+1)

           rvar[0]   = randomn(seed,1,/normal,/double)*sqrt(cnn[l])
           rvar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
           ivar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)

           cmbsignal[l,*] = dcomplex(rvar,ivar)
        endfor
        tmpalm.alm = cmbsignal
        mrs_almrec,tmpalm,newcmbobs12
        ud_grade,newcmbobs12,newcmbobs,order_in='NESTED',order_out='RING'
        
        for sc = 0,nscale-1 do begin
           newcmb[sc*npix:(sc+1)*npix-1] = mrs_wtmexhat(newcmbobs,scales[sc])
        endfor
        newcmb = newcmb + corrCoeff * wgal
     
        newCoeff = ( transpose(newcmb) # wgal )
        bootvar[iter] = newCoeff[0] / normGal
        
     endfor
     
     bootvarmean = mean(bootvar)
     corrVar = double(btstpvar) / double(btstpvar - 1.0d) * mean((bootvar - bootvarmean)^2)
     
  endif else corrVar = 1.0d / normGal
     
  if not keyword_set(noverb) then print,'Coefficient of correlation: ',corrCoeff
  if not keyword_set(noverb) then print,'Variance (bootstrap): ',corrVar
  zt = corrCoeff / sqrt(corrVar)
  if not keyword_set(noverb) then print,'Z-score: ',zt
  
  if bootstrap gt 0 then begin
 
     boottest = dblarr(bootstrap)
     for iter=0,bootstrap-1 do begin
        
        tmpalm  = icmbalm
        newcmb  = dblarr(nscale*npix)
        newcmb2 = dblarr(nscale*npix)
        
        cmbsignal = dcomplexarr(lmax+1,lmax+1)
        for l=lmin,lmax do begin
           rvar = dblarr(lmax+1)
           ivar = dblarr(lmax+1)
           
           rvar[0]   = randomn(seed,1,/normal,/double)*sqrt(cnn[l])
           rvar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
           ivar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
           
           cmbsignal[l,*] = dcomplex(rvar,ivar)
        endfor
        tmpalm.alm = cmbsignal
        mrs_almrec,tmpalm,newcmbobs12
        ud_grade,newcmbobs12,newcmbobs,order_in='NESTED',order_out='RING'
        
        for sc = 0,nscale-1 do begin
           newcmb[sc*npix:(sc+1)*npix-1] = mrs_wtmexhat(newcmbobs,scales[sc])
        endfor
        
        newcorrCoeff = ( transpose(newcmb) # wgal )
        
        if btstpvar gt 0 then begin
           
           bootvar = dblarr(btstpvar)
           for itvar=0,btstpvar-1 do begin
              
              cmbsignal = dcomplexarr(lmax+1,lmax+1)
              for l=lmin,lmax do begin
                 rvar = dblarr(lmax+1)
                 ivar = dblarr(lmax+1)
                 
                 rvar[0]   = randomn(seed,1,/normal,/double)*sqrt(cnn[l])
                 rvar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
                 ivar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
                 
                 cmbsignal[l,*] = dcomplex(rvar,ivar)
              endfor
              tmpalm.alm = cmbsignal
              mrs_almrec,tmpalm,newcmbobs12
              ud_grade,newcmbobs12,newcmbobs,order_in='NESTED',order_out='RING'
              
              for sc = 0,nscale-1 do begin
                 newcmb2[sc*npix:(sc+1)*npix-1] = mrs_wtmexhat(newcmbobs,scales[sc])
              endfor
              
              newcmb2 = newcmb2 + newcorrCoeff[0] / normGal * wgal
              
              newcorrCoeff2 = ( transpose(newcmb2) # wgal )
              bootvar[itvar] = newcorrcoeff2[0] / normGal
              
           endfor
           
           bootvarmean2 = mean(bootvar)
           corrVar2 = double(btstpvar) / double(btstpvar - 1.0d) * mean((bootvar - bootvarmean2)^2)
           
        endif else corrVar2 = 1.0d / normGal
        
        ;; Apply the test
        boottest[iter] = newcorrCoeff[0] / normGal / sqrt(corrVar2)
        
     endfor
  
     if keyword_set(nosmooth) then begin
        
        aix = where(boottest gt zt, count) ; Brutal (exact estimation of the p-value)
        
     endif  else begin 
        
        count     = 0.0d
        ;; bandwidth = 1.3167d * double(bootstrap)^(-4.0d / 9.0d) 
        bandwidth = 1.0d * double(bootstrap)^(-4.0d / 9.0d) 
        for iter=0ul,bootstrap-1 do count = count + gaussint((zt - boottest[iter])/bandwidth)
        
     endelse  
     
     pvalue = 1.0d - double(count)/double(bootstrap)
     rscore = inverf(1.0d - pvalue)*sqrt(2.0d)
     
     if not keyword_set(noverb) then print,'p-value: ',pvalue
     if not keyword_set(noverb) then print,'sigma: ',rscore
     
     return,{pvalue: pvalue, score: zt, real_score: rscore}

  endif

  return,{pvalue: 0.0d, score: zt, real_score: zt}

end

;
; Test if there is a correlation between the two input fields (inside the pixel domain)
; A double bootstrap (with a smooth bootstrap at the end for the pvalue) is performed 
; in order to be independent of cosmology hypothesis.
; INPUT: icmbobs:    the observed CMB field
;        igalobs:    the observed galaxy survey
;        bootstrap:  the number of bootstrap steps for the p-value estimation
;        btstpvar:   the number of bootstrap steps for the correlation variance estimation
;        mask:       the missing pixels (they will be avoided during the estimation)
;        nosmooth:   disable the kernel smoothing for the p-value estimation
;        noverb:     disable the printing of some information
; OUTPUT:
;        the p-value given the confidence for rejecting the null hypothesis (i.e. there is no correlation)
;
function pixelCorr,icmbobs,igalobs,bootstrap=bootstrap,btstpvar=btstpvar,mask=mask,nosmooth=nosmooth,noverb=noverb,$
                   lmin=lmin,lmax=lmax,cmbspec=cmbspec

  if not keyword_set(bootstrap) then bootstrap=199
  if not keyword_set(btstpvar)  then btstpvar=199

  ;; 1 - Create vector for the data manipulation (i.e. remove masked pixels)

  cmbpix = icmbobs
  galpix = igalobs

  if keyword_set(mask) then begin

     midx = where(mask eq 1.0d, count)
     cmbpix = icmbobs[midx]
     galpix = igalobs[midx]

  endif

  pixcount = n_elements(cmbpix)

  mrs_almtrans, icmbobs, icmbalm, /tab, /complex, lmax=lmax
 
  ;; 1bis - Do some estimations from perturbed observation
  ;; Use the theoritical spectra or estimate them

  cnn = dblarr(lmax+1)
  for l=0,lmax do begin
     vcnn = dblarr(2*l+1)
     for m=0,l do begin 
        vcnn[m]   = abs(icmbalm.alm[l,m])^2
        vcnn[l+m] = vcnn[m]     ; The m=0 value will be overwritten by m=l
     endfor
     cnn[l] = mean(vcnn)
  endfor

  if keyword_set(cmbspec) then cnn = cmbspec

  ;; 3 - Estimate correlation

  corrCoeff = ( transpose(cmbpix) # galpix )
  corrCoeff = corrCoeff[0]

  normGal   = ( transpose(galpix) # galpix )
  normGal   = normGal[0]

  corrCoeff = corrCoeff / normGal

  ;; 4 - Bootstrap for variance estimation (using parametric bootstrap)
  if btstpvar gt 0 then begin

     bootvar = dblarr(btstpvar)    
     tmpalm = icmbalm

     for iter=0,btstpvar-1 do begin
        
        cmbsignal = dcomplexarr(lmax+1,lmax+1)
        for l=0,lmax do begin
           rvar = dblarr(lmax+1)
           ivar = dblarr(lmax+1)

           rvar[0]   = randomn(seed,1,/normal,/double)*sqrt(cnn[l])
           rvar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
           ivar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)

           cmbsignal[l,*] = dcomplex(rvar,ivar)
        endfor

        tmpalm.alm = cmbsignal
        mrs_almrec,tmpalm,newcmbobs
        if keyword_set(mask) then newcmbobs = newcmbobs[midx]
        newcmbobs = newcmbobs + corrCoeff / normGal * galpix

        newCoeff = ( transpose(newcmbobs) # galpix )
        bootvar[iter] = newcoeff[0] / normGal
        
     endfor
     
     bootvarmean = mean(bootvar)
     corrVar = double(btstpvar) / double(btstpvar - 1.0d) * mean((bootvar - bootvarmean)^2)
     
  endif else corrVar = 1.0d / normGal[0]
     
  if not keyword_set(noverb) then print,'Coefficient of correlation: ',corrCoeff
  if not keyword_set(noverb) then print,'Variance (bootstrap): ',corrVar
  zt = corrCoeff / sqrt(corrVar)
  if not keyword_set(noverb) then print,'Z-score: ',zt
  
  ;; if not keyword_set(noverb) then print,'Coefficient of correlation: ',corrCoeff
  ;; zt = corrCoeff[0]
  
  boottest = dblarr(bootstrap)
  for iter=0,bootstrap-1 do begin
     
     tmpalm  = icmbalm
     cmbsignal = dcomplexarr(lmax+1,lmax+1)
     for l=0,lmax do begin
        rvar = dblarr(lmax+1)
        ivar = dblarr(lmax+1)

        rvar[0]   = randomn(seed,1,/normal,/double)*sqrt(cnn[l])
        rvar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
        ivar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)

        cmbsignal[l,*] = dcomplex(rvar,ivar)
     endfor

     tmpalm.alm = cmbsignal
     mrs_almrec,tmpalm,newcmbobs
     if keyword_set(mask) then newcmbobs = newcmbobs[midx]
     
     newcorrCoeff = ( transpose(newcmbobs) # galpix )
     
     if btstpvar gt 0 then begin
        
        bootvar = dblarr(btstpvar)
        for itvar=0,btstpvar-1 do begin
           
           cmbsignal = dcomplexarr(lmax+1,lmax+1)
           for l=0,lmax do begin
              rvar = dblarr(lmax+1)
              ivar = dblarr(lmax+1)
              
              rvar[0]   = randomn(seed,1,/normal,/double)*sqrt(cnn[l])
              rvar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
              ivar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
              
              cmbsignal[l,*] = dcomplex(rvar,ivar)
           endfor

           tmpalm.alm = cmbsignal
           mrs_almrec,tmpalm,newcmbobs
           if keyword_set(mask) then newcmbobs = newcmbobs[midx]
           
           newcmbobs = newcmbobs + newcorrCoeff[0] / normGal * galpix

           newcorrCoeff2 = ( transpose(newcmbobs) # galpix )
           bootvar[itvar] = newcorrcoeff2[0] / normGal
           
        endfor
        
        bootvarmean2 = mean(bootvar)
        corrVar2 = double(btstpvar) / double(btstpvar - 1.0d) * mean((bootvar - bootvarmean2)^2)
        
     endif else corrVar2 = 1.0d / normGal
     
     ;; Apply the test
     boottest[iter] = newcorrCoeff[0] / normGal / sqrt(corrVar2)
     
  endfor

  if keyword_set(nosmooth) then begin

     aix = where(boottest le zt, count) ; Brutal (exact estimation of the p-value)

  endif else begin

     count     = 0.0d
     ;; bandwidth = 1.3167d * double(bootstrap)^(-4.0d / 9.0d) 
     bandwidth = 1.0d * double(bootstrap)^(-4.0d / 9.0d) 
     for iter=0ul,bootstrap-1 do count = count + gaussint((zt - boottest[iter])/bandwidth)

  endelse
  
  pvalue = 1.0d - double(count)/double(bootstrap)
  rscore = inverf(1.0d - pvalue)*sqrt(2.0d)

  if not keyword_set(noverb) then print,'p-value: ',pvalue
  if not keyword_set(noverb) then print,'sigma: ',rscore

  return,{pvalue: pvalue, score: zt, real_score: rscore}

end

;
; Test if there is a correlation between the two input fields (inside the harmonic domain)
; A double bootstrap (with a smooth bootstrap at the end for the pvalue) is performed 
; in order to be independent of cosmology hypothesis.
; INPUT: icmbobs:    the observed CMB field
;        igalobs:    the observed galaxy survey
;        lmin:       the minimal pole
;        lmax:       the maximal pole
;        bootstrap:  the number of bootstrap steps for the p-value estimation
;        btstpvar:   the number of bootstrap steps for the correlation variance estimation
;        nosmooth:   disable the kernel smoothing for the p-value estimation
;        noverb:     disable the printing of some information
;        bootmet:    the bootstrap method (1 = parametric bootstrap, 2 = wild bootstrap, 3 = pair bootstrap)
; OUTPUT:
;        the p-value given the confidence for rejecting the null hypothesis (i.e. there is no correlation)
;
function harmonicCorr,icmbobs,igalobs,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btstpvar=btstpvar,nosmooth=nosmooth,noverb=noverb,$
                      bootmet=bootmet,cmbspec=cmbspec

  if not keyword_set(bootstrap) then bootstrap = 199
  if not keyword_set(btstpvar)  then btstpvar = 0
  if not keyword_set(bootmet)   then bootmet = 1 ; Parametric bootstrap by default

  ;; 1 - Create vector for the data manipulation

  mrs_almtrans, icmbobs, icmbalm, /tab, /complex, lmax=lmax
  mrs_almtrans, igalobs, ogalm, /tab, /complex, lmax=lmax

  almcount = 0ul
  for l=lmin,lmax do almcount = almcount+2*l+1ul

  cmbalm = dcomplexarr(almcount)
  galalm = dcomplexarr(almcount)

  ;; 1bis - Do some estimations from perturbed observation
  ;; Use the theoritical spectra or estimate them

  cnn = dblarr(lmax+1)
  
  for l=0,lmax do begin
     
     vcnn = dblarr(2*l+1)
     
     for m=0,l do begin 
        vcnn[m]   = abs(icmbalm.alm[l,m])^2
        vcnn[l+m] = vcnn[m]     ; The m=0 value will be overwritten by m=l
     endfor
     
     cnn[l] = mean(vcnn)
     
  endfor

  if keyword_set(cmbspec) then cnn = cmbspec

  ;; 2 - Put data into the big vectors

  iter = 0ul
  for l=lmin,lmax do begin
     for m=0,l do begin
        cmbalm[iter] = icmbalm.alm[l,m]
        galalm[iter] = ogalm.alm[l,m]
        iter = iter + 1ul
     endfor
     for m=1,l do begin
        cmbalm[iter] = icmbalm.alm[l,m]
        galalm[iter] = ogalm.alm[l,m]
        iter = iter + 1ul
     endfor
  endfor

  ;; 3 - Compute correlation coefficient (part one: scalar product)

  corrCoeff = real_part(transpose(conj(cmbalm))#galalm)
  normGal   = real_part(transpose(conj(galalm))#galalm)

  corrCoeff = corrCoeff[0] / normGal[0]

  ;; 4 - Monte Carlo or Parametric bootstrap for variance estimation
  if bootmet eq 1 then begin
    
     if btstpvar gt 0 then begin

        bootvar = dblarr(btstpvar)
        
        for iter=0,btstpvar-1 do begin
           
           cmbsignal = dcomplexarr(almcount)
           
           iternw = 0ul
           for l=lmin,lmax do begin
              
              rv0  = randomn(seed,1,/normal,/double)*sqrt(cnn[l]) + corrCoeff * real_part(ogalm.alm[l,*])
              rvar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d) + corrCoeff * real_part(ogalm.alm[l,*])
              ivar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d) + corrCoeff * imaginary(ogalm.alm[l,*])
              
              cmbsignal[iternw]            = rv0
              cmbsignal[iternw+1:iternw+l] = dcomplex(rvar,ivar)
              cmbsignal[iternw+l+1ul:iternw+2ul*l] = cmbsignal[iternw+1:iternw+l]
              
              iternw = iternw + 2 * l + 1ul
              
           endfor

           newcoeff      = real_part(transpose(conj(cmbsignal))#galalm)
           bootvar[iter] = newcoeff[0] / normGal[0]
           
        endfor
        
        bootvarmean = mean(bootvar)
        corrVar = double(btstpvar) / double(btstpvar - 1.0d) * mean((bootvar - bootvarmean)^2)

     endif else corrVar = 1.0d / normGal[0]
     
     if not keyword_set(noverb) then print,'Coefficient of correlation: ',corrCoeff
     if not keyword_set(noverb) then print,'Variance (bootstrap): ',corrVar
     zt = corrCoeff / sqrt(corrVar)
     if not keyword_set(noverb) then print,'Z-score: ',zt
          
     ;; if not keyword_set(noverb) then print,'Coefficient of correlation: ',corrCoeff
     ;; zt = corrCoeff[0]
     
     boottest = dblarr(bootstrap)
     for iter=0,bootstrap-1 do begin
        
        ;; New field under the null hypothesis
        cmbsignal = dcomplexarr(almcount)

        iternw = 0ul
        for l=lmin,lmax do begin

           rv0  = randomn(seed,1,/normal,/double)*sqrt(cnn[l])
           rvar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
           ivar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
           
           cmbsignal[iternw]            = rv0
           cmbsignal[iternw+1:iternw+l] = dcomplex(rvar,ivar)
           cmbsignal[iternw+l+1ul:iternw+2ul*l] = cmbsignal[iternw+1:iternw+l]
           
           iternw = iternw + 2 * l + 1ul
           
        endfor
        
        newcorrCoeff = real_part(transpose(conj(cmbsignal))#galalm)
        
        if btstpvar gt 0 then begin

           ;; Estimate the variance (nested bootstrap)
           bootvar = dblarr(btstpvar)
           for itvar=0,btstpvar-1 do begin
              
              cmbsignal2 = dcomplexarr(almcount)
              
              iternw = 0ul
              for l=lmin,lmax do begin
                 
                 rv0  = randomn(seed,1,/normal,/double)*sqrt(cnn[l]) + newcorrCoeff[0] / normGal[0] * real_part(ogalm.alm[l,*])
                 rvar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d) + newcorrCoeff[0] / normGal[0] * real_part(ogalm.alm[l,*])
                 ivar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d) + newcorrCoeff[0] / normGal[0] * imaginary(ogalm.alm[l,*])
                 
                 cmbsignal2[iternw]            = rv0
                 cmbsignal2[iternw+1:iternw+l] = dcomplex(rvar,ivar)
                 cmbsignal2[iternw+l+1ul:iternw+2ul*l] = cmbsignal2[iternw+1:iternw+l]
                 
                 iternw = iternw + 2 * l + 1ul
                 
              endfor

              newcoeff2      = real_part(transpose(conj(cmbsignal2))#galalm)
              bootvar[itvar] = newcoeff2[0] / normGal[0]
              
           endfor
           
           bootvarmean2 = mean(bootvar)
           corrVar2 = double(btstpvar) / double(btstpvar - 1.0d) * mean((bootvar - bootvarmean2)^2)
           
        endif else corrVar2 = 1.0d / normGal[0]
        
        ;; Apply the test
        boottest[iter] = newcorrCoeff[0] / normGal[0] / sqrt(corrVar2)

     endfor
     
  endif

  ;; Use wild bootstrap (non-parametric) in order to estimate the confidence interval
  if bootmet eq 2 then begin
     
     res = cmbalm - corrCoeff * galalm           ; This is residual under the null hypothesis
     hc3 = 1.0d / ( 1.0d - abs(galalm)^2 / normGal[0] ) ; HC3 (see papers)

     if btstpvar gt 0 then begin

        bootvar = dblarr(btstpvar)
        for iter=0ul,btstpvar-1 do begin
           
           ;; Wild sampling using Rademacher variables
           errs = double(randomn(seed,almcount,/uniform)) - 0.5d
           indx = where(errs eq 0.0d, count)
           if count gt 0.0 then errs[indx] = 1.0d
           errs = errs / abs(errs)
           
           ;; The new signal under alternative hypothesis
           cmbsignal     = corrCoeff * galalm + res * errs * hc3
           newcoeff      = real_part(transpose(conj(cmbsignal))#galalm)
           bootvar[iter] = newcoeff[0] / normGal[0]
           
        endfor
        
        bootvarmean = mean(bootvar)
        corrVar = double(btstpvar) / double(btstpvar - 1.0d) * mean((bootvar - bootvarmean)^2)

     endif else corrVar = 1.0d / normGal[0]
        
     if not keyword_set(noverb) then print,'Coefficient of correlation: ',corrCoeff
     if not keyword_set(noverb) then print,'Variance (bootstrap): ',corrVar
     zt = corrCoeff / sqrt(corrVar)
     if not keyword_set(noverb) then print,'Z-score: ',zt

     boottest = dblarr(bootstrap)
     
     ;; Compute the residuals (use for generating H0 samples)
     res = cmbalm ; This is residual under the null hypothesis
     
     for iter=0ul,bootstrap-1 do begin
        
        ;; Wild sampling using Rademacher variables
        errs = double(randomn(seed,almcount,/uniform)) - 0.5d
        indx = where(errs eq 0.0d, count)
        if count gt 0.0 then errs[indx] = 1.0d
        errs = errs / abs(errs)
        
        ;; The new signal under null hypothesis
        cmbsignal    = res * errs * hc3
        newcorrCoeff = real_part(transpose(conj(cmbsignal))#galalm)
        newcorrCoeff = newcorrCoeff[0] / normGal[0]
        
        ;; Get residual for the variance estimation
        resid = cmbsignal - newcorrCoeff * galalm

        if btstpvar gt 0 then begin

           ;; Estimate the variance (nested bootstrap)
           bootvar = dblarr(btstpvar)
           for itvar=0ul,btstpvar-1 do begin
              
              ;; Wild sampling using Rademacher variables
              errs = double(randomn(seed,almcount,/uniform)) - 0.5d
              indx = where(errs eq 0.0d, count)
              if count gt 0.0 then errs[indx] = 1.0d
              errs = errs / abs(errs)
              
              ;; The new signal under the current hypothesis
              cmbsignal = resid * errs * hc3 + newcorrCoeff * galalm
              newcoeff2 = real_part(transpose(conj(cmbsignal))#galalm)
              
              bootvar[itvar] = newcoeff2[0] / normGal[0]
              
           endfor
        
           bootvarmean2 = mean(bootvar)
           corrVar2 = double(btstpvar) / double(btstpvar - 1.0d) * mean((bootvar - bootvarmean2)^2)

        endif else corrVar2 = 1.0d / normGal[0]

        ;; Apply the test
        boottest[iter] = newcorrCoeff / sqrt(corrVar2)
        
     endfor
     
  endif

  if bootmet eq 3 then begin

     bootvar = dblarr(btstpvar)
     
     for iter=0ul,btstpvar-1 do begin
        
        ;; New sampling
        idx = ulong(almcount*randomn(seed,almcount,/uniform))
        
        ;; New coefficient of correlation
        newcoeff      = real_part(transpose(conj(cmbalm[idx]))#galalm[idx])
        newnormgal    = real_part(transpose(conj(galalm[idx]))#galalm[idx])
        
        bootvar[iter] = newcoeff[0] / newnormgal[0]
        
     endfor
     
     bootvarmean = mean(bootvar)
     corrVar = double(btstpvar) / double(btstpvar - 1.0d) * mean((bootvar - bootvarmean)^2)
     
     if not keyword_set(noverb) then print,'Coefficient of correlation: ',corrCoeff
     if not keyword_set(noverb) then print,'Variance (bootstrap): ',corrVar
     zt = corrCoeff / sqrt(corrVar)
     if not keyword_set(noverb) then print,'Z-score: ',zt
     
     ;; 5 - Bootstrap the test in order to estimate the p-value
     
     boottest = dblarr(bootstrap)
     for iter=0,bootstrap-1 do begin
        
        ;; New sampling
        idx = ulong(almcount*randomn(seed,almcount,/uniform))
        
        ;; New coefficient of correlation
        newcoeff   = real_part(transpose(conj(cmbalm[idx]))#galalm[idx])
        newnormgal = real_part(transpose(conj(galalm[idx]))#galalm[idx])
        
        newcoeff = newcoeff[0] / newnormgal[0]
        
        ;; Estimate the variance (nested bootstrap)
        cmbnew = cmbalm[idx]
        galnew = galalm[idx]
        
        bootvar = dblarr(btstpvar)
        
        for itvar=0ul,btstpvar-1 do begin
           
           ;; New sampling
           idx2 = ulong(almcount*randomn(seed,almcount,/uniform))
           
           ;; New coefficient of correlation
           newcoeff2      = real_part(transpose(conj(cmbnew[idx2]))#galnew[idx2])
           newnormgal2    = real_part(transpose(conj(galnew[idx2]))#galnew[idx2])
           
           bootvar[itvar] = newcoeff2[0] / newnormgal2[0]
           
        endfor
        
        bootvarmean2 = mean(bootvar)
        corrVar2 = double(btstpvar) / double(btstpvar - 1.0d) * mean((bootvar - bootvarmean2)^2)
        
        ;; Apply the test
        boottest[iter] = (newcoeff - corrCoeff)/sqrt(corrVar2)
        
     endfor
 
  endif

  if keyword_set(nosmooth) then begin

     aix = where(boottest le zt, count) ; Brutal (exact estimation of the p-value)

  endif else begin

     count     = 0.0d
     ;; bandwidth = 1.3167d * double(bootstrap)^(-4.0d / 9.0d) 
     bandwidth = 1.0d * double(bootstrap)^(-4.0d / 9.0d) 
     for iter=0ul,bootstrap-1 do count = count + gaussint((zt - boottest[iter])/bandwidth)

  endelse
  
  pvalue = 1.0d - double(count)/double(bootstrap)
  rscore = inverf(1.0d - pvalue)*sqrt(2.0d)

  if not keyword_set(noverb) then print,'p-value: ',pvalue
  if not keyword_set(noverb) then print,'sigma: ',rscore
  
  return,{pvalue: pvalue, score: zt, real_score: rscore}

end

;
; Detection of a correlation between LSS survey and CMB
; INPUT: icmbobs:  the observed CMB (suppose to be complete)
;        igalobs:  the galaxy survey used as a matter tracer
;        lmin:     the minimal l-pole to be considered (spherical harmonics specifics)
;        lmax:     the maximal l-pole "                (spherical harmonics specifics)
;        method:   the domain space (0: spherical harmonics, 1: pixels, 2: wavelets)
;        boostrap: number of bootstrap coefficients for p-value estimation
;        btstpvar: number of bootstrap coefficients for variance estimation
;        mask:     the joint mask LSS-CMB (pixels specific)
;        scales:   the scales (in arcmin) of the wavelets (wavelets specific)
;        nested:   set to true of the data are in nested format (wavelets specific)
;        nosmooth: disable the kernel smoothing for the p-value estimation
;        noverb:   disable the printing of some information
; OUTPUT: the result of the correlation with the corresponding score
;
function isw_corrTest,icmbobs,igalobs,lmin=lmin,lmax=lmax,method=method,bootstrap=bootstrap,btstpvar=btstpvar,mask=mask,scales=scales,nested=nested,$
                      noverb=noverb,nosmooth=nosmooth,nside=nside,usemc=usemc,bootmet=bootmet,cmbspec=cmbspec

  if not keyword_set(method) then method = 0

  if method eq 0 then $
     res = harmonicCorr(icmbobs,igalobs,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btstpvar=btstpvar,noverb=noverb,nosmooth=nosmooth,bootmet=bootmet,cmbspec=cmbspec)

  if method eq 1 then $
     res = pixelCorr(icmbobs,igalobs,bootstrap=bootstrap,btstpvar=btstpvar,mask=mask,noverb=noverb,nosmooth=nosmooth,lmin=lmin,lmax=lmax,cmbspec=cmbspec)

  if method eq 2 then $
     res = waveletCorr(icmbobs,igalobs,scales=scales,nested=nested,bootstrap=bootstrap,btstpvar=btstpvar,noverb=noverb,nosmooth=nosmooth,lmin=lmin,lmax=lmax,cmbspec=cmbspec)

  return,res

end
