;
; Correlation detection test. Bootstrap is performed for both variance and confidence interval estimation.
; Code for tomography context.
;
; AUTHOR: Francois-Xavier Dupe
; DATE: 24/06/11
; REVISION: 
;          

;
; The G(k,n,u) as defined in "Generalized FDR" article by Sarkat and Guo (2009)
; INPUT: k: the tolerance, i.e. the number of allowed least false rejection
;        n: the number of tests inside H0
;        u: variable
; OUTOUT: \tidle{G}_{k,n}
;
function f_gkn,k,n,u

  gsum = 0.0d

  for j = k,n do $
     gsum = gsum + exp(lngamma(n+1) - lngamma(j+1) - lngamma(n-j+1))*u^(double(j))*(1.0d - u)^(double((n-j)))

  return,gsum

end

;
; Test if there is a correlation between the two input fields (inside the harmonic domain)
; A double bootstrap (with a smooth bootstrap at the end for the pvalue) is performed 
; in order to be independent of cosmology hypothesis.
; INPUT: icmbobs:    the observed CMB field
;        igalobs:    the observed galaxy surveys
;        nfields:    the number of fields/surveys
;        lmin:       the minimal pole
;        lmax:       the maximal pole
;        bootstrap:  the number of bootstrap steps for the p-value estimation
;        btstpvar:   the number of bootstrap steps for the correlation variance estimation
;        nosmooth:   disable the kernel smoothing for the p-value estimation
;        noverb:     disable the printing of some information
; OUTPUT:
;        the p-value given the confidence for rejecting the null hypothesis (i.e. there is no correlation)
;
function harmonicCorrTomo,icmbobs,igalobs,nfields,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btstpvar=btstpvar,nosmooth=nosmooth,noverb=noverb,$
                          bootmet=bootmet,cmbspec=cmbspec,lambda=lambda,precision=precision,ktol=ktol

  if not keyword_set(bootstrap) then bootstrap = 199
  if not keyword_set(btstpvar)  then btstpvar = 0
  if not keyword_set(bootmet)   then bootmet = 1 ; Parametric bootstrap by default
  if not keyword_set(ktol)      then ktol = 1
  if not keyword_set(lambda)    then lambda = 0.9d
  if not keyword_set(precision) then precision = 10000ul

  ktol = fix(ktol)
  if ktol gt (lmax - lmin) then ktol = lmax - lmin
  if ktol lt 0 then ktol = 0

  ;; 0 - Initialize the vector for final step
  
  precision = ulong(3.0d * precision / lambda) ;; Upgrade the precision
  gidx = 3.0d / lambda * dindgen(precision) / double(precision)  ;; Generate an array with the good value
  gtab = gidx

  for j = 0ul,precision-1 do begin
     gtab[j] = gidx[j] * f_gkn(ktol-1,nfields-1,gidx[j])
  endfor

  ;; 1 - Create vector for the data manipulation

  mrs_almtrans, icmbobs, icmbalm, /tab, /complex, lmax=lmax

  ogalm = replicate(icmbalm,nfields) ;; Create an array of structures
  for nf=0,nfields-1 do begin
     mrs_almtrans, reform(igalobs[*,nf]), ogalm_a, /tab, /complex, lmax=lmax
     ogalm[nf] = ogalm_a
  endfor

  almcount = 0ul
  for l=lmin,lmax do almcount = almcount+2*l+1ul
  realcount = almcount ;; Saving the value (because of ktol)

  cmbalm = dcomplexarr(almcount)
  galalm = dcomplexarr(almcount,nfields)

  ;; 1bis - Do some estimations from perturbed observation
  ;; Use the theoritical spectra or estimate them

  cnn = dblarr(lmax+1)
  
  for l=0,lmax do begin
     vcnn = dblarr(2*l+1)
     for m=0,l do begin 
        vcnn[m]   = abs(icmbalm.alm[l,m])^2
        vcnn[l+m] = vcnn[m]     ; The m=0 value will be overwritten by m=l
     endfor
     cnn[l] = mean(vcnn)
  endfor

  if keyword_set(cmbspec) then cnn = cmbspec

  ;; 2 - Put data into the big vectors

  iter = 0ul
  for l=lmin,lmax do begin
     for m=0,l do begin
        cmbalm[iter] = icmbalm.alm[l,m]
        iter = iter + 1ul
     endfor
     for m=1,l do begin
        cmbalm[iter] = icmbalm.alm[l,m]
        iter = iter + 1ul
     endfor
  endfor

  for nf=0,nfields-1 do begin
     iter = 0ul
     for l=lmin,lmax do begin
        for m=0,l do begin
           galalm[iter,nf] = ogalm[nf].alm[l,m]
           iter = iter + 1ul
        endfor
        for m=1,l do begin
           galalm[iter,nf] = ogalm[nf].alm[l,m]
           iter = iter + 1ul
        endfor
     endfor
  endfor

  ;; 3 - Compute correlation coefficient for each survey (part one: scalar product)

  corrCoeff = dblarr(nfields)
  for nf=0,nfields-1 do begin
     
     corrC = real_part(transpose(conj(cmbalm))#reform(galalm[*,nf]))
     normG = real_part(transpose(conj(reform(galalm[*,nf])))#reform(galalm[*,nf]))
     
     corrCoeff[nf] = corrC[0] / normG[0]
     
  endfor

  ;; 4 - Monte Carlo or Parametric bootstrap for variance estimation

  if bootmet eq 1 then begin
    
     bootvar = dblarr(btstpvar,nfields)
     for iter=0,btstpvar-1 do begin
        
        for nf=0,nfields-1 do begin
           cmbsignal = dcomplexarr(realcount)
           
           iternw = 0ul
           for l=lmin,lmax do begin
              
              rv0  = randomn(seed,1,/normal,/double)*sqrt(cnn[l]) + corrCoeff[nf] * real_part(reform(ogalm[nf].alm[l,*]))
              rvar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d) + corrCoeff[nf] * real_part(reform(ogalm[nf].alm[l,*]))
              ivar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d) + corrCoeff[nf] * imaginary(reform(ogalm[nf].alm[l,*]))
              
              cmbsignal[iternw] = rv0
              cmbsignal[iternw+1:iternw+l] = dcomplex(rvar,ivar)
              cmbsignal[iternw+l+1ul:iternw+2ul*l] = cmbsignal[iternw+1:iternw+l]
              
              iternw = iternw + 2 * l + 1ul
              
           endfor

           newcoeff = real_part(transpose(conj(cmbsignal))#reform(galalm[*,nf]))
           normGal  = real_part(transpose(conj(reform(galalm[*,nf])))#reform(galalm[*,nf]))
           bootvar[iter,nf] = newcoeff[0] / normGal[0]

        endfor
        
     endfor
     
     ;; Compute the variance of the sum method (aka: method 1)

     corrVar     = dblarr(nfields)
     for nf=0,nfields-1 do begin
        bootvarmean = mean(reform(bootvar[*,nf]))
        corrVar[nf] = double(btstpvar) / double(btstpvar - 1.0d) * mean((reform(bootvar[*,nf]) - bootvarmean)^2)
     endfor

     allCorrCoeff = corrCoeff / sqrt(corrVar)

     ;; bootvarmean = mean(bootvar)
     ;; corrVar = double(btstpvar) / double(btstpvar - 1.0d) * mean((bootvar - bootvarmean)^2)
     
     if not keyword_set(noverb) then print,'Coefficients of correlation: ',corrCoeff
     if not keyword_set(noverb) then print,'Variance (bootstrap): ',corrVar
     zt = allCorrCoeff
     if not keyword_set(noverb) then print,'Z-score: ',zt
          
     boottest = dblarr(bootstrap,nfields)
     for iter=0,bootstrap-1 do begin
        
        ;; New field under the null hypothesis
        cmbsignal = dcomplexarr(realcount)

        iternw = 0ul
        for l=lmin,lmax do begin

           rv0  = randomn(seed,1,/normal,/double)*sqrt(cnn[l])
           rvar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
           ivar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d)
           
           cmbsignal[iternw]            = rv0
           cmbsignal[iternw+1:iternw+l] = dcomplex(rvar,ivar)
           cmbsignal[iternw+l+1ul:iternw+2ul*l] = cmbsignal[iternw+1:iternw+l]
           
           iternw = iternw + 2 * l + 1ul
           
        endfor

        newcorrCoeff = dblarr(nfields)
        for nf=0,nfields-1 do begin
           
           newcorrC = real_part(transpose(conj(cmbsignal))#reform(galalm[*,nf]))
           newnormG = real_part(transpose(conj(reform(galalm[*,nf])))#reform(galalm[*,nf]))
           
           newcorrCoeff[nf] = newcorrC[0] / newnormG[0]
           
        endfor        
     
        bootvar = dblarr(btstpvar,nfields)
        for itvar=0,btstpvar-1 do begin
           
           for nf=0,nfields-1 do begin
              cmbsignal2 = dcomplexarr(realcount)
              
              iternw = 0ul
              for l=lmin,lmax do begin
                 
                 rv0  = randomn(seed,1,/normal,/double)*sqrt(cnn[l]) + newcorrCoeff[nf] * real_part(reform(ogalm[nf].alm[l,*]))
                 rvar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d) + newcorrCoeff[nf] * real_part(reform(ogalm[nf].alm[l,*]))
                 ivar = randomn(seed,l,/normal,/double)*sqrt(cnn[l]/2.0d) + newcorrCoeff[nf] * imaginary(reform(ogalm[nf].alm[l,*]))
                 
                 cmbsignal2[iternw]            = rv0
                 cmbsignal2[iternw+1:iternw+l] = dcomplex(rvar,ivar)
                 cmbsignal2[iternw+l+1ul:iternw+2ul*l] = cmbsignal2[iternw+1:iternw+l]
                 
                 iternw = iternw + 2 * l + 1ul
                 
              endfor

              newcoeff = real_part(transpose(conj(cmbsignal2))#reform(galalm[*,nf]))
              normGal  = real_part(transpose(conj(reform(galalm[*,nf])))#reform(galalm[*,nf]))
              bootvar[itvar,nf] = newcoeff[0] / normGal[0]

           endfor
           
        endfor

        corrVar2 = dblarr(nfields)
        for nf=0,nfields-1 do begin
           bootvarmean2 = mean(reform(bootvar[*,nf]))
           corrVar2[nf] = double(btstpvar) / double(btstpvar - 1.0d) * mean((reform(bootvar[*,nf]) - bootvarmean2)^2)
        endfor
        
        
        ;; Apply the test
        boottest[iter,*] = newcorrCoeff / sqrt(corrVar2)

     endfor
     
  endif

  count = dblarr(nfields)
  if keyword_set(nosmooth) then begin

     for nf = 0,nfields-1 do begin
        aix = where(reform(boottest[*,nf]) le reform(zt[nf]), ct) ; Brutal (exact estimation of the p-value)
        count[nf] = ct
     endfor

  endif else begin

     for nf = 0,nfields-1 do begin
        ct = 0.0d
        ;; bandwidth = 1.3167d * double(bootstrap)^(-4.0d / 9.0d) 
        bandwidth = 1.0d * double(bootstrap)^(-4.0d / 9.0d) 
        for iter=0ul,bootstrap-1 do ct = ct + gaussint(reform(zt[nf] - boottest[iter,nf])/bandwidth)
        count[nf] = ct
     endfor

  endelse

  pvalues = 1.0d - double(count)/double(bootstrap)

  ;; Using classical testing based on optimal false detection rate
  idx     = sort(pvalues)
  spv     = pvalues[idx]

  alpha     = dindgen(precision) * lambda / double(precision)
  bestalpha = 1.0d ;; Nothing by default

  for ipa = 0ul,precision-1 do begin

     ;; First stage: Compute j, i.e. the test threshold
     jtest  = 0
     for i = 0,nfields-1 do if spv[i] le lambda then jtest = i

     ;; Second stage: seek for the rejected tests
     lval = -1
     for i = 0,jtest do begin
        
        cval = ((1+i)*alpha[ipa]*(1.0d - lambda))/(lambda*double(nfields - jtest))
        gval = 1.0d

        for j = 0ul,precision-1 do begin
           if gtab[j] ge cval then begin
              gval = gidx[j]
              break
           endif
        endfor

        if gval gt 1.0d then gval = 1.0d
        if lambda*gval ge spv[i] then lval = i

     endfor

     if lval ne -1 then begin
        bestalpha = alpha[ipa]
        break
     endif
     
  endfor

  if not keyword_set(noverb) then print,(lval+1),' test(s) has been rejected with k-FDR = ',bestalpha
  pvalue = bestalpha

  rscore = inverf(1.0d - pvalue)*sqrt(2.0d)

  if not keyword_set(noverb) then print,'p-value: ',pvalue
  if not keyword_set(noverb) then print,'sigma: ',rscore
  
  return,{pvalue: pvalue, score: rscore, real_score: rscore}

end

;
; Detection of a correlation between LSS survey and CMB
; INPUT: icmbobs:  the observed CMB (suppose to be complete)
;        igalobs:  the galaxy surveys used as a matter tracer
;        nfields:  the number of surveys/fields
;        lmin:     the minimal l-pole to be considered (spherical harmonics specifics)
;        lmax:     the maximal l-pole "                (spherical harmonics specifics)
;        method:   the domain space (0: spherical harmonics, 1: pixels, 2: wavelets)
;        boostrap: number of bootstrap coefficients for p-value estimation
;        btstpvar: number of bootstrap coefficients for variance estimation
;        mask:     the joint mask LSS-CMB (pixels specific)
;        scales:   the scales (in arcmin) of the wavelets (wavelets specific)
;        nested:   set to true of the data are in nested format (wavelets specific)
;        nosmooth: disable the kernel smoothing for the p-value estimation
;        noverb:   disable the printing of some information
; OUTPUT: the result of the correlation with the corresponding score
;
function isw_corrTestTomo,icmbobs,igalobs,nfields,lmin=lmin,lmax=lmax,method=method,bootstrap=bootstrap,btstpvar=btstpvar,mask=mask,scales=scales,nested=nested,$
                         noverb=noverb,nosmooth=nosmooth,nside=nside,usemc=usemc,bootmet=bootmet,cmbspec=cmbspec,lambda=lambda,precision=precision,ktol=ktol

  if not keyword_set(method) then method = 0

  if method eq 0 then $
     res = harmonicCorrTomo(icmbobs,igalobs,nfields,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btstpvar=btstpvar,noverb=noverb,nosmooth=nosmooth,bootmet=bootmet,cmbspec=cmbspec,lambda=lambda,precision=precision,ktol=ktol)

  ;; if method eq 1 then $
  ;;    res = pixelCorr(icmbobs,igalobs,bootstrap=bootstrap,btstpvar=btstpvar,mask=mask,noverb=noverb,nosmooth=nosmooth,lmin=lmin,lmax=lmax,cmbspec=cmbspec)

  ;; if method eq 2 then $
  ;;    res = waveletCorr(icmbobs,igalobs,scales=scales,nested=nested,bootstrap=bootstrap,btstpvar=btstpvar,noverb=noverb,nosmooth=nosmooth,lmin=lmin,lmax=lmax,cmbspec=cmbspec)

  return,res

end
