;
; Signal detection using an amplitude estimation following by a test on the nullity of the estimate.
; In order to infer the confidence on the test, we perform a bootstrap or Monte-Carlo method.
; Remark that under Gaussian assomption, we can also compute the p-value using Student-t distribution.
;
; AUTHOR: Francois-Xavier Dupe
; DATE: 27/08/10
; REVISION: 
;          

;
; Amplitude estimation inside the wavelet domain
; icmbobs:    the observed CMB (suppose to be complete)
; igalobs:    the galaxy survey used as a matter tracer (suppose to be bias corrected)
; cmbspec:    the power spectra of the primordial CMB (must have the required size)
; galspec:    the power spectra of the matter map
; iswspec:    the power spectra of the expected ISW signal
; lmin:       the minimal l-pole to be considered
; lmax:       the maximal l-pole "
; bootstrap:  number of boostrap/MC iterations for the confidence intervals of the test
; btspvar:    if not null, use btspvar bootstrap iteration for variance estimation instead of using the Cramer-Rao bound
; noverb:     remove verbose display
; nosmooth:   do not smooth the empirical distribution (may lead to inaccurate result)
; usemc:      use on Monte-Carlo for the H_0 distribution instead on the boostrap
;
function waveletAmpl,icmbobs,igalobs,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btspvar=btspvar,noverb=noverb,$
                     nosmooth=nosmooth,usemc=usemc,ktol=ktol,nested=nested,scales=scales

  if not keyword_set(bootstrap) then bootstrap=0
  if not keyword_set(btspvar)   then btspvar=0
  if not keyword_set(bootmet)   then bootmet=1 ; By default use parametric bootstrap
  if not keyword_set(ktol)      then ktol=0.0d

  npix   = n_elements(icmbobs)
  nside  = npix2nside(npix)
  nscale = n_elements(scales)

  ;; 1 - Create vector for the data manipulation

  mrs_almtrans, icmbobs, icmbalm, /tab, /complex, lmax=lmax
  mrs_almtrans, igalobs, ogalm, /tab, /complex, lmax=lmax

  ;; 2 - Do some estimations from perturbed observation
  ;; Use the theoritical spectra or estimate them
  
  cgg = dblarr(lmax+1)
  cgt = dblarr(lmax+1)
  cnn = dblarr(lmax+1)
   
  for l=0,lmax do begin
     
     vcgg = dblarr(2*l+1)
     vcgt = dblarr(2*l+1)
     vcnn = dblarr(2*l+1)
     
     for m=0,l do begin 
        vcgg[m] = abs(ogalm.alm[l,m])^2
        vcgt[m] = real_part(ogalm.alm[l,m]*conj(icmbalm.alm[l,m]))
        vcnn[m] = abs(icmbalm.alm[l,m])^2
     endfor
     for m=1,l do begin 
        vcgg[l+m] = abs(ogalm.alm[l,m])^2
        vcgt[l+m] = real_part(ogalm.alm[l,m]*conj(icmbalm.alm[l,m]))
        vcnn[l+m] = abs(icmbalm.alm[l,m])^2
     endfor
     
     cgg[l] = mean(vcgg)
     cgt[l] = mean(vcgt)
     cnn[l] = mean(vcnn)
     
  endfor
  
  ;; Keep the parametric CNN
  cnnest = cnn

  if keyword_set(cmbspec) then cnn=cmbspec
  if keyword_set(galspec) then cgg=galspec
  if keyword_set(iswspec) then cgt=iswspec

  iiswalm = ogalm
  for l=0,lmin-1  do iiswalm.alm[l,*] *= 0.0d
  for l=lmin,lmax do iiswalm.alm[l,*] *= cgt[l]/cgg[l] ; Apply the Fc filter
  
  ;; ISW without Ctt dependancy
  iiswalm_ctt = iiswalm
  for l=lmin,lmax do iiswalm_ctt.alm[l,*] /= cnn[l] ; Apply the Fc filter

  mrs_almrec,iiswalm,iiswobs
  mrs_almrec,iiswalm_ctt,iiswobs_ctt

  ;; 3 - Put data into the big vectors
  cmbobs     = icmbobs
  galobs     = igalobs
  iswobs     = iiswobs
  iswobs_ctt = iiswobs_ctt

  if keyword_set(nested) then begin
     ud_grade,icmbobs,cmbobs,order_in='NESTED',order_out='RING'
     ud_grade,igalobs,galobs,order_in='NESTED',order_out='RING'
     ud_grade,iiswobs,iswobs,order_in='NESTED',order_out='RING'
     ud_grade,iiswobs_ctt,iswobs_ctt,order_in='NESTED',order_out='RING'
  endif

  wcmb     = dblarr(nscale*npix)
  wgal     = dblarr(nscale*npix)
  wisw     = dblarr(nscale*npix)
  wisw_ctt = dblarr(nscale*npix)

  for sc = 0,nscale-1 do begin
     wcmb[sc*npix:(sc+1)*npix-1]     = mrs_wtmexhat(cmbobs,scales[sc])
     wgal[sc*npix:(sc+1)*npix-1]     = mrs_wtmexhat(galobs,scales[sc])
     wisw[sc*npix:(sc+1)*npix-1]     = mrs_wtmexhat(iswobs,scales[sc])
     wisw_ctt[sc*npix:(sc+1)*npix-1] = mrs_wtmexhat(iswobs_ctt,scales[sc])
  endfor

  ;; 4 - Test the current model with the data

  denom = real_part(transpose(wcmb) # wisw_ctt)
  divis = real_part(transpose(wisw) # wisw_ctt)
  
  vari1 = 1.0d / divis
  corr1 = denom / divis

  if not keyword_set(noverb) then print,'Alpha: ',corr1
  if not keyword_set(noverb) then print,'Variance: ',vari1

  ;; 5 - Apply the z-score
  
  if not keyword_set(noverb) then print,'z-test for alpha=0'
  zt = corr1/sqrt(vari1)
  if not keyword_set(noverb) then print,'test: ',zt

  if bootstrap gt 0 then begin

     ;; Use Monte-Carlo for estimating the confidence intervals
     if keyword_set(usemc) then begin
        
        boottest = dblarr(bootstrap)
        zt2      = zt[0]
        divis    = real_part(transpose(wisw) # wisw_ctt)
        
        for iter=0ul,bootstrap-1ul do begin
           
           ;; New field under the null hypothesis
           cmbsignal = icmbalm
           for l=lmin,lmax do begin
              rvar = dblarr(lmax+1)
              ivar = dblarr(lmax+1)
              
              rvar[0]   = randomn(seed,1,/normal,/double)*sqrt(cmbspec[l])
              rvar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cmbspec[l]/2.0d)
              ivar[1:l] = randomn(seed,l,/normal,/double)*sqrt(cmbspec[l]/2.0d)
              
              cmbsignal.alm[l,*] = dcomplex(rvar,ivar)
           endfor

           mrs_almrec,cmbsignal,cmbnew

           wcmbnew = dblarr(nscale*npix)
           for sc = 0,nscale-1 do begin
              wcmbnew[sc*npix:(sc+1)*npix-1] = mrs_wtmexhat(cmbnew,scales[sc])
           endfor

           ;; Compute estimator
           denom = real_part(transpose(wcmbnew) # wisw_ctt)

           ;; Perform test
           boottest[iter] = denom[0] / sqrt(divis[0])
           
        endfor
        
     endif

     ;; Use parametric bootstrap in order to estimate the confidence interval
     if not keyword_set(usemc) then begin

        boottest = dblarr(bootstrap)
        zt2 = zt[0]
        
        for iter=0ul,bootstrap-1ul do begin
           
           ;; New field under the null hypothesis
           cmbsignal = icmbalm
           for l=lmin,lmax do begin

              rv0  = randomn(seed,1,/normal,/double)*sqrt(cnnest[l])
              rvar = randomn(seed,l,/normal,/double)*sqrt(cnnest[l]/2.0d)
              ivar = randomn(seed,l,/normal,/double)*sqrt(cnnest[l]/2.0d)              

              cmbsignal.alm[l,0]   = rv0
              cmbsignal.alm[l,1:l] = dcomplex(rvar,ivar)
           endfor

           mrs_almres,cmbsignal,cmbnew

           wcmbnew = dblarr(nscale*npix)
           for sc = 0,nscale-1 do begin
              wcmbnew[sc*npix:(sc+1)*npix-1] = mrs_wtmexhat(cmbnew,scales[sc])
           endfor

           ;; Compute estimator
           denom = real_part(transpose(wcmbnew) # wisw_ctt)
           
           ;; Perform test
           boottest[iter] = denom[0] / sqrt(divis[0])
           
        endfor

     endif

     if keyword_set(nosmooth) then begin
        
        aix = where(boottest le zt2, count) ; Brutal (exact estimation of the p-value)
              
     endif else begin
        
        count     = 0.0d
        ;; bandwidth = 1.3167d * double(bootstrap)^(-4.0d / 9.0d) 
        bandwidth = 1.0d * double(bootstrap)^(-4.0d / 9.0d) 
        for iter=0ul,bootstrap-1ul do count = count + gaussint((zt2 - boottest[iter])/bandwidth)
        
     endelse
     
     pvalue = 1.0d - double(count)/double(bootstrap)
     rscore = inverf(1.0d - pvalue)*sqrt(2.0d)

     if not keyword_set(noverb) then print,'p-value:',pvalue
     if not keyword_set(noverb) then print,'sigma: ',rscore
     
     return,{pvalue: pvalue, score: zt2, real_score: rscore }
     
  endif
  
  return,{ mean: corr1, var: vari1, score: zt2 }
  
end

;
; Amplitude estimation inside the spherical harmonic domain
; icmbobs:    the observed CMB (suppose to be complete)
; igalobs:    the galaxy survey used as a matter tracer (suppose to be bias corrected)
; cmbspec:    the power spectra of the primordial CMB (must have the required size)
; galspec:    the power spectra of the matter map
; iswspec:    the power spectra of the expected ISW signal
; lmin:       the minimal l-pole to be considered
; lmax:       the maximal l-pole "
; bootstrap:  number of boostrap/MC iterations for the confidence intervals of the test
; btspvar:    if not null, use btspvar bootstrap iteration for variance estimation instead of using the Cramer-Rao bound
; noverb:     remove verbose display
; nosmooth:   do not smooth the empirical distribution (may lead to inaccurate result)
; usemc:      use on Monte-Carlo for the H_0 distribution instead on the boostrap
; bootmet:    the boostrap method (1 = parametric bootstrap, 2 = wild boostrap, 3 = pair boostrap)
;
function harmonicAmpl,icmbobs,igalobs,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btspvar=btspvar,noverb=noverb,$
                      nosmooth=nosmooth,usemc=usemc,bootmet=bootmet

  if not keyword_set(bootstrap) then bootstrap=0
  if not keyword_set(btspvar)   then btspvar=0
  if not keyword_set(bootmet)   then bootmet=1 ; By default use parametric bootstrap
  if not keyword_set(ktol)      then ktol=0.0d

  ;; 1 - Create vector for the data manipulation

  mrs_almtrans, icmbobs, icmbalm, /tab, /complex, lmax=lmax
  mrs_almtrans, igalobs, ogalm, /tab, /complex, lmax=lmax

  almcount = 0ul
  for l=lmin,lmax do almcount = almcount+2*l+1ul
  realcount = almcount ;; Saving the value
  
  cmbalm = dcomplexarr(almcount)
  iswalm = dcomplexarr(almcount)
  galalm = dcomplexarr(almcount)
  cmbvar = dblarr(almcount)
  galvar = dblarr(almcount)
  iswvar = dblarr(almcount)

  ;; 2 - Do some estimations from perturbed observation
  ;; Use the theoritical spectra or estimate them
  
  cgg = dblarr(lmax+1)
  cgt = dblarr(lmax+1)
  cnn = dblarr(lmax+1)
   
  for l=0,lmax do begin
     
     vcgg = dblarr(2*l+1)
     vcgt = dblarr(2*l+1)
     vcnn = dblarr(2*l+1)
     
     for m=0,l do begin 
        vcgg[m] = abs(ogalm.alm[l,m])^2
        vcgt[m] = real_part(ogalm.alm[l,m]*conj(icmbalm.alm[l,m]))
        vcnn[m] = abs(icmbalm.alm[l,m])^2
     endfor
     for m=1,l do begin 
        vcgg[l+m] = abs(ogalm.alm[l,m])^2
        vcgt[l+m] = real_part(ogalm.alm[l,m]*conj(icmbalm.alm[l,m]))
        vcnn[l+m] = abs(icmbalm.alm[l,m])^2
     endfor
     
     cgg[l] = mean(vcgg)
     cgt[l] = mean(vcgt)
     cnn[l] = mean(vcnn)
     
  endfor
  
  ;; Keep the parametric CNN
  cnnest = cnn

  if keyword_set(cmbspec) then cnn=cmbspec
  if keyword_set(galspec) then cgg=galspec
  if keyword_set(iswspec) then cgt=iswspec

  iiswalm = ogalm
  for l=0,lmin-1  do iiswalm.alm[l,*] *= 0.0d
  for l=lmin,lmax do iiswalm.alm[l,*] *= cgt[l]/cgg[l] ; Apply the Fc filter

  ;; 3 - Put data into the big vectors
  iter = 0ul
  for l=lmin,lmax do begin
     for m=0,l do begin
        cmbalm[iter] = icmbalm.alm[l,m]
        iswalm[iter] = iiswalm.alm[l,m]
        galalm[iter] = ogalm.alm[l,m]
        cmbvar[iter] = cnn[l]
        galvar[iter] = cgg[l]
        iswvar[iter] = cgt[l]
        iter = iter + 1ul
     endfor
     for m=1,l do begin
        cmbalm[iter] = icmbalm.alm[l,m]
        iswalm[iter] = iiswalm.alm[l,m]
        galalm[iter] = ogalm.alm[l,m]
        cmbvar[iter] = cnn[l]
        galvar[iter] = cgg[l]
        iswvar[iter] = cgt[l]
        iter = iter + 1ul
     endfor
  endfor

  ;; Experimental part
  if ktol gt 0.0 then begin

     misw    = abs(iswalm)
     mad_isw = mad(misw)
     gind    = where(misw gt ktol*mad_isw, count)
     print,'Keeping only ',count,' elements.'

     iswalm   = iswalm[gind]
     galalm   = galalm[gind]
     cmbalm   = cmbalm[gind]
     cmbvar   = cmbvar[gind]
     galvar   = galvar[gind]
     iswvar   = iswvar[gind]
     amlcount = count

  endif

  ;; 4 - Test the current model with the data

  denom = real_part(transpose(conj(cmbalm)) # (iswalm / cmbvar))
  divis = real_part(transpose(conj(iswalm)) # (iswalm / cmbvar))
  
  vari1 = 1.0d / divis
  corr1 = denom / divis

  if not keyword_set(noverb) then print,'Alpha: ',corr1
  if not keyword_set(noverb) then print,'Variance: ',vari1

; 4 - Apply the z-score
  
  if not keyword_set(noverb) then print,'z-test for alpha=0'
  zt = corr1/sqrt(vari1)
  if not keyword_set(noverb) then print,'test: ',zt

  if bootstrap gt 0 then begin

     ;; Use Monte-Carlo for estimating the confidence intervals
     if keyword_set(usemc) then begin
        
        boottest = dblarr(bootstrap)
        zt2 = zt[0]
        
        for iter=0ul,bootstrap-1ul do begin
           
           ;; New field under the null hypothesis
           cmbsignal = dcomplexarr(realcount)
           iternw = 0ul
           for l=lmin,lmax do begin
              
              rv0  = randomn(seed,1,/normal,/double)*sqrt(cmbspec[l])
              rvar = randomn(seed,l,/normal,/double)*sqrt(cmbspec[l]/2.0d)
              ivar = randomn(seed,l,/normal,/double)*sqrt(cmbspec[l]/2.0d)
              
              cmbsignal[iternw]            = rv0
              cmbsignal[iternw+1:iternw+l] = dcomplex(rvar,ivar)
              cmbsignal[iternw+l+1ul:iternw+2ul*l] = cmbsignal[iternw+1:iternw+l]
              
              iternw = iternw + 2 * l + 1ul
              
           endfor

           if ktol gt 0.0d then cmbsignal = cmbsignal[gind]

           ;; Compute estimator
           denom = real_part(transpose(conj(iswalm)) # (cmbsignal / cmbvar))
           divis = real_part(transpose(conj(iswalm)) # (iswalm / cmbvar))
           
           ;; Perform test
           boottest[iter] = denom[0] / sqrt(divis[0])
           
        endfor
        
     endif

     ;; Use parametric bootstrap in order to estimate the confidence interval
     if not keyword_set(usemc) and bootmet eq 1 then begin

        boottest = dblarr(bootstrap)
        zt2 = zt[0]
        
        for iter=0ul,bootstrap-1ul do begin
           
           ;; New field under the null hypothesis
           cmbsignal = dcomplexarr(realcount)
           iternw = 0ul
           for l=lmin,lmax do begin
              
              rv0  = randomn(seed,1,/normal,/double)*sqrt(cnnest[l])
              rvar = randomn(seed,l,/normal,/double)*sqrt(cnnest[l]/2.0d)
              ivar = randomn(seed,l,/normal,/double)*sqrt(cnnest[l]/2.0d)
              
              cmbsignal[iternw]            = rv0
              cmbsignal[iternw+1:iternw+l] = dcomplex(rvar,ivar)
              cmbsignal[iternw+l+1ul:iternw+2ul*l] = cmbsignal[iternw+1:iternw+l]
              
              iternw = iternw + 2 * l + 1ul
              
           endfor

           if ktol gt 0.0d then cmbsignal = cmbsignal[gind]

           ;; Compute estimator
           denom = real_part(transpose(conj(iswalm)) # (cmbsignal / cmbvar))
           divis = real_part(transpose(conj(iswalm)) # (iswalm / cmbvar))
           
           ;; Perform test
           boottest[iter] = denom[0] / sqrt(divis[0])
           
        endfor

     endif

     ;; Use wild bootstrap (non-parametric) in order to estimate the confidence interval
     if not keyword_set(usemc) and bootmet eq 2 then begin

        boottest = dblarr(bootstrap)
        zt2 = zt[0]
        iswnorm = real_part(transpose(conj(iswalm)) # iswalm)
        hc3 = 1.0d / (1.0d - abs(iswalm)^2 / iswnorm[0])
        
        ;; Compute the residuals (use for generating H0 samples)
        res = cmbalm ; This is the residual under the null hypothesis
        
        for iter=0ul,bootstrap-1ul do begin

           ;; Wild sampling using Rademacher variables
           errs = double(randomn(seed,almcount,/uniform)) - 0.5d
           indx = where(errs eq 0.0d, count)
           if count gt 0.0 then errs[indx] = 1.0d
           errs = errs / abs(errs)
          
           ;; The new signal under null hypothesis
           cmbsignal = res * errs * hc3

           ;; Compute estimator
           denom = real_part(transpose(conj(iswalm)) # (cmbsignal / cmbvar))
           divis = real_part(transpose(conj(iswalm)) # (iswalm / cmbvar))
           
           ;; Perform test
           boottest[iter] = denom[0] / sqrt(divis[0])
           
        endfor

     endif

     if not keyword_set(usemc) and bootmet eq 3 then begin
        
        if btspvar gt 0 then begin
           
           ;; Bootstrap zone (variance estimation)
           boottest  = dblarr(btspvar)
           for iter=0,btspvar-1 do begin
              
              ;; New sampling
              nidx = ulong(almcount*randomn(seed,almcount,/uniform))
              
              ;; Compute estimator
              denom = real_part(transpose(conj(iswalm[nidx])) # (cmbalm[nidx] / cmbvar[nidx]))
              divis = real_part(transpose(conj(iswalm[nidx])) # (iswalm[nidx] / cmbvar[nidx]))
              
              ;; Perform test
              boottest[iter] = denom / divis
              
           endfor
           
           bootmean = mean(boottest)
           bootvar  = double(btspvar) / (btspvar - 1.0d) * mean((boottest - bootmean)^2)
           
        endif else bootvar = 1.0d / divis[0]
        
        zt2 = corr1[0] / sqrt(bootvar)
        if not keyword_set(noverb) then print,'test (variance bootstrap): ',zt2
        
        ;; Bootstrap zone (Hypothesis testing)
        cmbsignal = cmbalm      ; - corr1[0] * iswalm ; Remove ISW signal and whitened
        iswsignal = iswalm
        
        ;; 1 - Bootstrap the test under H0
        boottest  = dblarr(bootstrap)
        for iter=0,bootstrap-1 do begin
           
           ;; New sampling
           nidx = ulong(almcount*randomn(seed,almcount,/uniform))
           
           ;; Compute estimator
           denom = real_part(transpose(conj(iswsignal[nidx])) # (cmbsignal[nidx] / cmbvar[nidx]))
           divis = real_part(transpose(conj(iswsignal[nidx])) # (iswsignal[nidx] / cmbvar[nidx]))
                      
           if btspvar gt 0 then begin
              
              ;; Nested bootstrap for variance estimation
              boottest2 = dblarr(btspvar)
              cmb2      = cmbsignal[nidx]
              isw2      = iswsignal[nidx]
              cmbvar2   = cmbvar[nidx]
              
              for nestiter=0,btspvar-1 do begin
                 
                 ;; New sampling
                 nidx2 = ulong(almcount*randomn(seed,almcount,/uniform))
                 
                 ;; Compute estimator
                 denom2 = real_part(transpose(conj(isw2[nidx2])) # (cmb2[nidx2] / cmbvar2[nidx2]))
                 divis2 = real_part(transpose(conj(isw2[nidx2])) # (isw2[nidx2] / cmbvar2[nidx2]))
                 
                 ;; Perform test
                 boottest2[nestiter] = denom2 / divis2
                 
              endfor
              bootmean2 = mean(boottest2)
              bootvar2  = double(btspvar) / (btspvar - 1.0d) * mean((boottest2 - bootmean2)^2)
              
           endif else bootvar2 = 1.0d / divis[0]
           
           ;; Perform test
           boottest[iter] = ( denom[0] / divis[0] - corr1[0]) / sqrt(bootvar2)
           
        endfor
        
     endif
     
     ;; Boottest gives an estimate of the EDF (using an histogram)
     ;; Save these data into a file
     ;; openw,1,'bootEDF_2Mass.dat'
     ;; for iter=0ul,bootstrap-1 do printf,1,format='(D)',boottest[iter]
     ;; close,1

     if keyword_set(nosmooth) then begin
        
        aix = where(boottest le zt2, count) ; Brutal (exact estimation of the p-value)
              
     endif else begin
        
        count     = 0.0d
        ;; bandwidth = 1.3167d * double(bootstrap)^(-4.0d / 9.0d) 
        bandwidth = 1.0d * double(bootstrap)^(-4.0d / 9.0d) 
        for iter=0ul,bootstrap-1ul do count = count + gaussint((zt2 - boottest[iter])/bandwidth)
        
     endelse
     
     pvalue = 1.0d - double(count)/double(bootstrap)
     rscore = inverf(1.0d - pvalue)*sqrt(2.0d)

     if not keyword_set(noverb) then print,'p-value:',pvalue
     if not keyword_set(noverb) then print,'sigma: ',rscore
     
     return,{pvalue: pvalue, score: zt2, real_score: rscore }
     
  endif
  
  return,{ mean: corr1, var: vari1, score: zt2 }
  
end

;
; Estimation of the amplitude of the ISW signal inside the given CMB
; INPUT:
; icmbobs:    the observed CMB (suppose to be complete)
; igalobs:    the galaxy survey used as a matter tracer (suppose to be bias corrected)
; cmbspec:    the power spectra of the primordial CMB (must have the required size)
; galspec:    the power spectra of the matter map
; iswspec:    the power spectra of the expected ISW signal
; lmin:       the minimal l-pole to be considered
; lmax:       the maximal l-pole "
; method:     select the domain space (only spherical harmonics are available for the moment)
; bootstrap:  number of boostrap/MC iterations for the confidence intervals of the test
; btspvar:    if not null, use btspvar bootstrap iteration for variance estimation instead of using the Cramer-Rao bound
; noverb:     remove verbose display
; nosmooth:   do not smooth the empirical distribution (may lead to inaccurate result)
; usemc:      use on Monte-Carlo for the H_0 distribution instead on the boostrap
;
function isw_estAmpl,icmbobs,igalobs,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,method=method,$
                       bootstrap=bootstrap,btspvar=btspvar,noverb=noverb,nosmooth=nosmooth,usemc=usemc,bootmet=bootmet,$
                       nested=nested,scales=scales

  if not keyword_set(method) then method = 0

  if method eq 0 then $
     res = harmonicAmpl(icmbobs,igalobs,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btspvar=btspvar,noverb=noverb,$
                        nosmooth=nosmooth,usemc=usemc,bootmet=bootmet)
  if method eq 1 then $
     res = waveletAmpl(icmbobs,igalobs,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btspvar=btspvar,noverb=noverb,$
                       nosmooth=nosmooth,usemc=usemc,nested=nested,scales=scales)
  return,res

end
