;
; Signal detection using an amplitude estimation following by a test on the nullity of the estimate.
; In order to infer the confidence on the test, we perform a pair bootstrap.
; This is the tomographic version.
;
; AUTHOR: Francois-Xavier Dupe
; DATE: 27/06/11
; REVISION: 
;

;
; Amplitude estimation inside the spherical harmonic domain in tomographic case
; icmbobs:    the observed CMB (suppose to be complete)
; igalobs:    the galaxy surveys used as a matter tracer (suppose to be bias corrected)
; nfields:    the number of fields
; cmbspec:    the power spectra of the primordial CMB (must have the required size)
; galspec:    the power spectra of the different of the matter maps
; iswspec:    the power spectra of the different of the expected ISW signal
; lmin:       the minimal l-pole to be considered
; lmax:       the maximal l-pole "
; bootstrap:  number of boostrap/MC iterations for the confidence intervals of the test
; btspvar:    if not null, use btspvar bootstrap iteration for variance estimation instead of using the Cramer-Rao bound
; noverb:     remove verbose display
; nosmooth:   do not smooth the empirical distribution (may lead to inaccurate result)
; usemc:      use on Monte-Carlo for the H_0 distribution instead on the boostrap
;
function harmonicAmplTomo,icmbobs,igalobs,nfields,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btspvar=btspvar,$
                          noverb=noverb,nosmooth=nosmooth,usemc=usemc,bootmet=bootmet

  if not keyword_set(bootstrap) then bootstrap=0
  if not keyword_set(btspvar)   then btspvar=0
  if not keyword_set(bootmet)   then bootmet=1 ; By default use parametric bootstrap

  ;; 1 - Create vector for the data manipulation

  mrs_almtrans, icmbobs, icmbalm, /tab, /complex, lmax=lmax

  ogalm = replicate(icmbalm,nfields) ;; Create an array of structures
  for nf=0,nfields-1 do begin
     mrs_almtrans, reform(igalobs[*,nf]), ogalm_a, /tab, /complex, lmax=lmax
     ogalm[nf] = ogalm_a
  endfor

  almcount = 0ul
  for l=lmin,lmax do almcount = almcount+2*l+1ul
  realcount = almcount ;; Saving the value
  
  cmbalm = dcomplexarr(almcount)
  iswalm = dcomplexarr(almcount)
  cmbvar = dblarr(almcount)
  iswvar = dblarr(almcount)

  ;; 2 - Do some estimations from perturbed observation
  ;; Use the theoritical spectra or estimate them
  
  cgg = dblarr(lmax+1,nfields)
  cgt = dblarr(lmax+1,nfields)
  cnn = dblarr(lmax+1)
   
  for l=0,lmax do begin
     vcnn = dblarr(2*l+1)
     for m=0,l do begin 
        vcnn[m] = abs(icmbalm.alm[l,m])^2
     endfor
     for m=1,l do begin
        vcnn[l+m] = abs(icmbalm.alm[l,m])^2
     endfor
     cnn[l] = mean(vcnn)
  endfor

  for nf=0,nfields-1 do begin
     for l=0,lmax do begin
        
        vcgg = dblarr(2*l+1)
        vcgt = dblarr(2*l+1)
        
        for m=0,l do begin 
           vcgg[m] = abs(ogalm[nf].alm[l,m])^2
           vcgt[m] = real_part(ogalm[nf].alm[l,m]*conj(icmbalm.alm[l,m]))
        endfor
        for m=1,l do begin 
           vcgg[l+m] = abs(ogalm[nf].alm[l,m])^2
           vcgt[l+m] = real_part(ogalm[nf].alm[l,m]*conj(icmbalm.alm[l,m]))
        endfor
        
        cgg[l,nf] = mean(vcgg)
        cgt[l,nf] = mean(vcgt)
        
     endfor
  endfor
  
  ;; Keep the parametric CNN
  cnnest = cnn

  if keyword_set(cmbspec) then cnn=cmbspec
  if keyword_set(galspec) then cgg=galspec
  if keyword_set(iswspec) then cgt=iswspec

  iiswalm = ogalm

  for nf=0,nfields-1 do begin
     for l=0,lmin-1  do iiswalm[nf].alm[l,*] *= 0.0d
     for l=lmin,lmax do iiswalm[nf].alm[l,*] *= cgt[l,nf]/cgg[l,nf] ; Apply the Fc filter
  endfor

  ;; 3 - Put data into the big vectors
  iter = 0ul
  for l=lmin,lmax do begin
     for m=0,l do begin
        cmbalm[iter] = icmbalm.alm[l,m]
        cmbvar[iter] = cnn[l]
        iter = iter + 1ul
     endfor
     for m=1,l do begin
        cmbalm[iter] = icmbalm.alm[l,m]
        cmbvar[iter] = cnn[l]
        iter = iter + 1ul
     endfor
  endfor

  for nf=0,nfields-1 do begin
     iter = 0ul
     for l=lmin,lmax do begin
        for m=0,l do begin
           iswalm[iter] = iswalm[iter] + iiswalm[nf].alm[l,m]
           iswvar[iter] = iswvar[iter] + cgt[l,nf]
           iter = iter + 1ul
        endfor
        for m=1,l do begin
           iswalm[iter] = iswalm[iter] + iiswalm[nf].alm[l,m]
           iswvar[iter] = iswvar[iter] + cgt[l,nf]
           iter = iter + 1ul
        endfor
     endfor
  endfor

  ;; 4 - Test the current model with the data

  denom = real_part(transpose(conj(cmbalm)) # (iswalm / cmbvar))
  divis = real_part(transpose(conj(iswalm)) # (iswalm / cmbvar))
  
  vari1 = 1.0d     / divis[0]
  corr1 = denom[0] / divis[0]
  
  if not keyword_set(noverb) then print,'Alpha: ',corr1
  if not keyword_set(noverb) then print,'Variance: ',vari1

; 4 - Apply the z-score
  
  if not keyword_set(noverb) then print,'z-test for alpha=0'
  zt = corr1/sqrt(vari1)
  if not keyword_set(noverb) then print,'test: ',zt

  if bootstrap gt 0 then begin

     ;; Use Monte-Carlo for estimating the confidence intervals
     if keyword_set(usemc) then begin
        
        boottest = dblarr(bootstrap)
        zt2 = zt[0]
        
        for iter=0ul,bootstrap-1ul do begin
           
           ;; New field under the null hypothesis
           cmbsignal = dcomplexarr(realcount)
           iternw = 0ul
           for l=lmin,lmax do begin
              
              rv0  = randomn(seed,1,/normal,/double)*sqrt(cmbspec[l])
              rvar = randomn(seed,l,/normal,/double)*sqrt(cmbspec[l]/2.0d)
              ivar = randomn(seed,l,/normal,/double)*sqrt(cmbspec[l]/2.0d)
              
              cmbsignal[iternw]              = rv0
              cmbsignal[iternw+1ul:iternw+l] = dcomplex(rvar,ivar)
              cmbsignal[iternw+l+1ul:iternw+2ul*l] = cmbsignal[iternw+1:iternw+l]
              
              iternw = iternw + 2 * l + 1ul
              
           endfor

           ;; Compute estimator
           denom = real_part(transpose(conj(iswalm)) # (cmbsignal / cmbvar))
           divis = real_part(transpose(conj(iswalm)) # (iswalm / cmbvar))
           
           ;; Perform test
           boottest[iter] = denom[0] / sqrt(divis[0])
           
        endfor
        
     endif

     ;; Use parametric bootstrap in order to estimate the confidence interval
     if not keyword_set(usemc) and bootmet eq 1 then begin

        boottest = dblarr(bootstrap)
        zt2 = zt[0]
        
        for iter=0ul,bootstrap-1ul do begin
           
           ;; New field under the null hypothesis
           cmbsignal = dcomplexarr(realcount)
           iternw = 0ul
           for l=lmin,lmax do begin
              
              rv0  = randomn(seed,1,/normal,/double)*sqrt(cnnest[l])
              rvar = randomn(seed,l,/normal,/double)*sqrt(cnnest[l]/2.0d)
              ivar = randomn(seed,l,/normal,/double)*sqrt(cnnest[l]/2.0d)
              
              cmbsignal[iternw]              = rv0
              cmbsignal[iternw+1ul:iternw+l] = dcomplex(rvar,ivar)
              cmbsignal[iternw+l+1ul:iternw+2ul*l] = cmbsignal[iternw+1:iternw+l]
              
              iternw = iternw + 2 * l + 1ul
              
           endfor

           ;; Compute estimator
           denom = real_part(transpose(conj(iswalm)) # (cmbsignal / cmbvar))
           divis = real_part(transpose(conj(iswalm)) # (iswalm / cmbvar))
           
           ;; Perform test
           boottest[iter] = denom[0] / sqrt(divis[0])
           
        endfor

     endif

     ;; Use wild bootstrap (non-parametric) in order to estimate the confidence interval
     if not keyword_set(usemc) and bootmet eq 2 then begin

        boottest = dblarr(bootstrap)
        zt2 = zt[0]
        iswnorm = real_part(transpose(conj(iswalm)) # iswalm)
        hc3 = 1.0d / (1.0d - abs(iswalm)^2 / iswnorm[0])
        
        ;; Compute the residuals (use for generating H0 samples)
        res = cmbalm ; This is the residual under the null hypothesis
        
        for iter=0ul,bootstrap-1ul do begin

           ;; Wild sampling using Rademacher variables
           errs = double(randomn(seed,almcount,/uniform)) - 0.5d
           indx = where(errs eq 0.0d, count)
           if count gt 0.0 then errs[indx] = 1.0d
           errs = errs / abs(errs)
          
           ;; The new signal under null hypothesis
           cmbsignal = res * errs * hc3

           ;; Compute estimator
           denom = real_part(transpose(conj(iswalm)) # (cmbsignal / cmbvar))
           divis = real_part(transpose(conj(iswalm)) # (iswalm / cmbvar))
           
           ;; Perform test
           boottest[iter] = denom[0] / sqrt(divis[0])
           
        endfor

     endif
     
     ;; Boottest gives an estimate of the EDF (using an histogram)
     ;; Save these data into a file
     ;; openw,1,'bootEDF_2Mass.dat'
     ;; for iter=0ul,bootstrap-1 do printf,1,format='(D)',boottest[iter]
     ;; close,1

     if keyword_set(nosmooth) then begin
        
        aix = where(boottest le zt2, count) ; Brutal (exact estimation of the p-value)
              
     endif else begin
        
        count     = 0.0d
        ;; bandwidth = 1.3167d * double(bootstrap)^(-4.0d / 9.0d) 
        bandwidth = 1.0d * double(bootstrap)^(-4.0d / 9.0d) 
        for iter=0ul,bootstrap-1ul do count = count + gaussint((zt2 - boottest[iter])/bandwidth)
        
     endelse
     
     pvalue = 1.0d - double(count)/double(bootstrap)
     rscore = inverf(1.0d - pvalue)*sqrt(2.0d)

     if not keyword_set(noverb) then print,'p-value:',pvalue
     if not keyword_set(noverb) then print,'sigma: ',rscore
     
     return,{pvalue: pvalue, score: zt2, real_score: rscore }
     
  endif
  
  return,{ mean: corr1, var: vari1, score: zt2 }
  
end

;
; Estimation of the amplitude of the ISW signal inside the given CMB
; INPUT:
; icmbobs:    the observed CMB (suppose to be complete)
; igalobs:    the galaxy surveys used as a matter tracer (suppose to be bias corrected)
; cmbspec:    the power spectra of the primordial CMB (must have the required size)
; galspec:    the power spectra of the different matter maps
; iswspec:    the power spectra of the difference expected ISW signals
; lmin:       the minimal l-pole to be considered
; lmax:       the maximal l-pole "
; method:     select the domain space (only spherical harmonics are available for the moment)
; bootstrap:  number of boostrap/MC iterations for the confidence intervals of the test
; btspvar:    if not null, use btspvar bootstrap iteration for variance estimation instead of using the Cramer-Rao bound
; noverb:     remove verbose display
; nosmooth:   do not smooth the empirical distribution (may lead to inaccurate result)
; usemc:      use on Monte-Carlo for the H_0 distribution instead on the boostrap
;
function isw_estAmplTomo,icmbobs,igalobs,nfields,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,method=method,$
                       bootstrap=bootstrap,btspvar=btspvar,noverb=noverb,nosmooth=nosmooth,usemc=usemc,bootmet=bootmet

  if not keyword_set(method) then method = 0

  if method eq 0 then $
     res = harmonicAmplTomo(icmbobs,igalobs,nfields,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btspvar=btspvar,noverb=noverb,$
                            nosmooth=nosmooth,usemc=usemc,bootmet=bootmet)
  ;; if method eq 1 then $
  ;;    res = waveletAmplNew(icmbobs,igalobs,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btspvar=btspvar,noverb=noverb,$
  ;;                         nosmooth=nosmooth,usemc=usemc,ktol=ktol,nested=nested,scales=scales)

  return,res

end
