;;
;; Testing performance of the Tomographic Sackay Method using Monte-Carlo.
;; We use the theoretical Euclid power spectra.
;; Author: Francois-Xavier Dupe
;; Version: 1.0 - written for the comparison of the inpainting technics
;;              - we compare two differents inpainting methods
;; 
;; .r isw_corrTest.pro
;; .r isw_estAmplNew.pro
;; .r isw_powspTest.pro
;; .r isw_testModel.pro
;; .r isw_powspTest.pro
;; .r isw_fullSaclayTest.pro

; Convert a vector of p-value into sigma
function computeEquivSigma,tab

  ;; plimit = 0.000001d
  tsiz = n_elements(tab)
  ntab = dblarr(tsiz)
  for itr=0ul,tsiz-1ul do begin

     ntab[itr] = inverf(1.0 - tab[itr])*sqrt(2.0)
     ;; if tab[itr] le plimit then ntab[itr] = inverf(plimit)/sqrt(2.0)

  endfor

  return,ntab

end

; Test each part of the Saclay method using bith Gaussian and log-normal matter density fields
pro isw_fullTomoTest

  ;; Initialize some important constants
  MC        = 100
  lmin      = 2
  lmax      = 100 ;; The testing limit
  slmax     = 140 ;; Use for generating the maps
  bootstrap = 10000
  nbbins    = 10
  retsav    = 0
  initmc    = 1
  itermc    = 2000
  slmax     = 400
  nside     = 64 ; Current nside
  npix      = nside2npix(nside)
  savecurr  = 'euclid_tomo_curr_071011.xdr'
  savefinal = 'euclid_tomo_final_071011.xdr'

  ;; Fix decision parameters
  pthres     = 0.045d  ; Threshold for the model testing p-value
  psigma     = 2.0d    ; Threshold on the detection sigma for the others methods

  ;; Store the results of the tests of each possibility [ method, inpainting, noisw ]
  pvalue_tab = dblarr(MC,3,2,2)
  score_tab  = dblarr(MC,3,2,2)

  ;; Restore the power spectra
  restore,'~/IDL/Work/wmapiswspec-2mass.xdr',/verb     ;; Get the CMB power spectra
  restore,'1107_Euclid_tomography_withbias.sav',/verb  ;; Get the gal/ISW power spectra

  ;; Compute the power spectra for a fixed sample of the multipoles
  galspec = dblarr(slmax+1,nbbins)
  iswspec = dblarr(slmax+1,nbbins)

  for i=0,nbbins-1 do begin
     ;; SA part only good for the largest scales (l > 50)
     galspec[1:slmax,i] = interpol(reform(cgg_lin_sa.cl[*,i]),cgg_lin_sa.l,fix(findgen(slmax)+1))
     iswspec[1:slmax,i] = interpol(reform(cgt_lin_sa.cl[*,i]),cgt_lin_sa.l,fix(findgen(slmax)+1))
  endfor

  ;; Get the masks needed for the test

  ;; WMAP mask
  cmbmask = rims('~/IDL/Data/wmap_temperature_analysis_mask_r9_7yr_v4.fits')
  cmbmask = cmbmask[*,0]

  ud_grade,cmbmask,cmbmaskll,nside_out=nside,order_in='NESTED'
  cmbmask = cmbmaskll
  aix = where(cmbmask ne 1.0)
  cmbmask[aix] = 0.0

  ;; Perform the main MC iteration
  for i=1,MC do begin

     ;; Restore old data if needed
     if retsav eq 1 then begin

        restore,savecurr
        retsav = 0   ; Disable the restore step
        i = i + 1    ; Update the counter to its real value

     endif

     ;; A new simulation
     print,'Main MC ',i

     ;; 1- Generate the new CMB primordial field using the power spectra
     imatnocor = isw_newsimu(cmbspec,lmax=slmax,nside=nside)
    
     ;; Beware Tomographic way if generating the ISW/Galaxy galaxy overdensity field.
     imatisw_bins = dblarr(npix,nbbins)
     imagcor_bins = dblarr(npix,nbbins)
     ;; g_poisson_bn = dblarr(npix,nbbins)

     for j=0,nbbins-1 do begin

        ;; 2- Generate the log-normal galaxy field using the power spectra
        imagcor_bins[*,j] = isw_newsimu(reform(galspec[*,j]),/lognormal,lmax=slmax,nside=nside)
        
        ;; Create observation fields
        ;; g_true    = long64((reform(imagcor_bins[*,j]) + 1.0d) * 40.0d)
        ;; g_poisson = g_true
        ;; for ki=0ul,npix-1 do if g_true[ki] le 0 then g_poisson[ki] = 0 else g_poisson[ki] = randomn(seed,poisson=g_true[ki])
        ;; g_poisson_bn[*,j] = g_poisson

        ;; 3- Generate the ISW field using the matter density field

        ;; Log-normal field
        mrs_almtrans,reform(imagcor_bins[*,j]),iswalm,/tab,/complex,lmax=slmax
        iswalm.alm[0,*] = 0.0d     ;; Remove the mean value
        mrs_almrec,iswalm,imagcor  ;; Save the new map
        for l=1,slmax do iswalm.alm[l,*] = iswalm.alm[l,*] * iswspec[l,j] / galspec[l,j]
        mrs_almrec,iswalm,imatisw_tmp

        imatisw_bins[*,j] = imatisw_tmp / 1d6 ;; Use the same unit everywhere

     endfor

     imatisw = dblarr(npix) * 0.0d
     for j=0,nbbins-1 do imatisw = imatisw + reform(imatisw_bins[*,j])

     stop

     ;; 4- Add ISW to CMB (with the tomography)

     imatcor = imatnocor + imatisw

     ;; Inpainting of the CMB overdensity (with and without ISW)
     cmbinpaint       = mrs_alm_inpainting(imatcor*cmbmask,mask=cmbmask,opt='-i40 -v')
     cmbinpaint_noisw = mrs_alm_inpainting(imatnocor*cmbmask,mask=cmbmask,opt='-i40 -v')

     ;; stop

     ;; --- Amplitude Estimation ---

     ;; Amplitude tests with complete data
     print,'Amplitude tests with complete data'
     t1 = isw_estAmplTomo(imatcor,imagcor_bins,nbbins,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btspvar=0,bootmet=1,/usemc)
    
     pvalue_tab(i-1,0,0,0) = t1.pvalue
     score_tab(i-1,0,0,0)  = t1.score

     t2 = isw_estAmplTomo(imatnocor,imagcor_bins,nbbins,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btspvar=0,bootmet=1,/usemc)
    
     pvalue_tab(i-1,0,0,1) = t2.pvalue
     score_tab(i-1,0,0,1)  = t2.score

     ;; Amplitude tests with inpainted data
     print,'Amplitude tests with inpainted data'
     t3 = isw_estAmplTomo(cmbinpaint,imagcor_bins,nbbins,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btspvar=0,bootmet=1,/usemc)
    
     pvalue_tab(i-1,0,1,0) = t3.pvalue
     score_tab(i-1,0,1,0)  = t3.score

     t4 = isw_estAmplTomo(cmbinpaint_noisw,imagcor_bins,nbbins,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,btspvar=0,bootmet=1,/usemc)
    
     pvalue_tab(i-1,0,1,1) = t4.pvalue
     score_tab(i-1,0,1,1)  = t4.score

     ;; --- Model selection ---

     ;; Model tests with complete data
     print,'Model tests with complete data'
     t5 = isw_testModelTomo(imatcor,imagcor_bins,nbbins,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap) 

     pvalue_tab(i-1,1,0,0) = t5.pvalue
     score_tab(i-1,1,0,0)  = t5.score

     t6 = isw_testModelTomo(imatnocor,imagcor_bins,nbbins,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap) 

     pvalue_tab(i-1,1,0,1) = t6.pvalue
     score_tab(i-1,1,0,1)  = t6.score

     ;; Model tests with inpainted data
     print,'Model tests with inpainted data'
     t7 = isw_testModelTomo(cmbinpaint,imagcor_bins,nbbins,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap) 

     pvalue_tab(i-1,1,1,0) = t7.pvalue
     score_tab(i-1,1,1,0)  = t7.score

     t8 = isw_testModelTomo(cmbinpaint_noisw,imagcor_bins,nbbins,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap) 

     pvalue_tab(i-1,1,1,1) = t8.pvalue
     score_tab(i-1,1,1,1)  = t8.score

     ;; --- Correlation detection ---

     ;; Correlation test (harmonic space version) with complete data
     print,'Correlation test (harmonic space version) with complete data'
     t9 = isw_corrTestTomo(imatcor,imagcor_bins,nbbins,method=0,lmin=lmin,lmax=lmax,bootstrap=200,btstpvar=101,bootmet=1)

     pvalue_tab(i-1,2,0,0) = t9.pvalue
     score_tab(i-1,2,0,0)  = t9.score

     t10 = isw_corrTestTomo(imatnocor,imagcor_bins,nbbins,method=0,lmin=lmin,lmax=lmax,bootstrap=200,btstpvar=101,bootmet=1)

     pvalue_tab(i-1,2,0,1) = t10.pvalue
     score_tab(i-1,2,0,1)  = t10.score

     ;; Correlation test (harmonic space version) with inpainted data
     print,'Correlation test (harmonic space version) with inpainted data'
     t11 = isw_corrTestTomo(cmbinpaint,imagcor_bins,nbbins,method=0,lmin=lmin,lmax=lmax,bootstrap=200,btstpvar=101,bootmet=1)

     pvalue_tab(i-1,2,1,0) = t11.pvalue
     score_tab(i-1,2,1,0)  = t11.score

     t12 = isw_corrTestTomo(imatnocor,imagcor_bins,nbbins,method=0,lmin=lmin,lmax=lmax,bootstrap=200,btstpvar=101,bootmet=1)

     pvalue_tab(i-1,2,1,1) = t12.pvalue
     score_tab(i-1,2,1,1)  = t12.score
   
     ;; Intermediate saves
     save,/variables,filename=savecurr

  endfor

  ;; Compute the type I and II errors (false positive and false negative)
  ;; This will help us to compare "objectively" the tests

  ;; Amplitude estimation
  print,'---- Amplitude tests ----'
 
  print,'With complete data (log-normal)'
  idxfn = where(pvalue_tab[*,0,0,0] gt pthres, countfn )
  tfn   = double(countfn) / double(MC) 
  idxfp = where(pvalue_tab[*,0,0,1] lt pthres, countfp )
  tfp   = double(countfp) / double(MC) 
  print,'False positive: ',tfp,' False negative: ',tfn
  print,'Mean sigma (with ISW): ',mean(score_tab[*,0,0,0]),' +- ',stddev(score_tab[*,0,0,0])
  print,'Mean sigma (no ISW): ',mean(score_tab[*,0,0,1]),' +- ',stddev(score_tab[*,0,0,1])
  print,'Mean p-value (with ISW): ',mean(pvalue_tab[*,0,0,0]),' +- ',stddev(pvalue_tab[*,0,0,0])
  print,'Mean p-value (no ISW): ',mean(pvalue_tab[*,0,0,1]),' +- ',stddev(pvalue_tab[*,0,0,1])
  tsigm = computeEquivSigma(reform(pvalue_tab[*,0,0,0]))
  print,'Mean corrected sigma (with ISW): ',mean(tsigm),' +- ',stddev(tsigm)
  tsigm = computeEquivSigma(reform(pvalue_tab[*,0,0,1]))
  print,'Mean corrected sigma (no ISW): ',mean(tsigm),' +- ',stddev(tsigm)

  print,'With inpainting (log-normal)'
  idxfn = where(pvalue_tab[*,0,1,0] gt pthres, countfn )
  tfn   = double(countfn) / double(MC) 
  idxfp = where(pvalue_tab[*,0,1,1] lt pthres, countfp )
  tfp   = double(countfp) / double(MC) 
  print,'False positive: ',tfp,' False negative: ',tfn
  print,'Mean sigma (with ISW): ',mean(score_tab[*,0,1,0]),' +- ',stddev(score_tab[*,0,1,0])
  print,'Mean sigma (no ISW): ',mean(score_tab[*,0,1,1]),' +- ',stddev(score_tab[*,0,1,1])
  print,'Mean p-value (with ISW): ',mean(pvalue_tab[*,0,1,0]),' +- ',stddev(pvalue_tab[*,0,1,0])
  print,'Mean p-value (no ISW): ',mean(pvalue_tab[*,0,1,1]),' +- ',stddev(pvalue_tab[*,0,1,1])
  tsigm = computeEquivSigma(reform(pvalue_tab[*,0,1,0]))
  print,'Mean corrected sigma (with ISW): ',mean(tsigm),' +- ',stddev(tsigm)
  tsigm = computeEquivSigma(reform(pvalue_tab[*,0,1,1]))
  print,'Mean corrected sigma (no ISW): ',mean(tsigm),' +- ',stddev(tsigm)

   ;; Models tests
  print,'---- Model selection tests ----'

  print,'With complete data (log-normal)'
  idxfn = where(pvalue_tab[*,1,0,0] gt pthres, countfn )
  tfn   = double(countfn) / double(MC) 
  idxfp = where(pvalue_tab[*,1,0,1] lt pthres, countfp )
  tfp   = double(countfp) / double(MC) 
  print,'False positive: ',tfp,' False negative: ',tfn
  print,'Mean sigma (with ISW): ',mean(score_tab[*,1,0,0]),' +- ',stddev(score_tab[*,1,0,0])
  print,'Mean sigma (no ISW): ',mean(score_tab[*,1,0,1]),' +- ',stddev(score_tab[*,1,0,1])
  print,'Mean p-value (with ISW): ',mean(pvalue_tab[*,1,0,0]),' +- ',stddev(pvalue_tab[*,1,0,0])
  print,'Mean p-value (no ISW): ',mean(pvalue_tab[*,1,0,1]),' +- ',stddev(pvalue_tab[*,1,0,1])
  tsigm = computeEquivSigma(reform(pvalue_tab[*,1,0,0]))
  print,'Mean corrected sigma (with ISW): ',mean(tsigm),' +- ',stddev(tsigm)
  tsigm = computeEquivSigma(reform(pvalue_tab[*,1,0,1]))
  print,'Mean corrected sigma (no ISW): ',mean(tsigm),' +- ',stddev(tsigm)

  print,'With inpainting (log-normal)'
  idxfn = where(pvalue_tab[*,1,1,0] gt pthres, countfn )
  tfn   = double(countfn) / double(MC) 
  idxfp = where(pvalue_tab[*,1,1,1] lt pthres, countfp )
  tfp   = double(countfp) / double(MC) 
  print,'False positive: ',tfp,' False negative: ',tfn
  print,'Mean sigma (with ISW): ',mean(score_tab[*,1,1,0]),' +- ',stddev(score_tab[*,1,1,0])
  print,'Mean sigma (no ISW): ',mean(score_tab[*,1,1,1]),' +- ',stddev(score_tab[*,1,1,1])
  print,'Mean p-value (with ISW): ',mean(pvalue_tab[*,1,1,0]),' +- ',stddev(pvalue_tab[*,1,1,0])
  print,'Mean p-value (no ISW): ',mean(pvalue_tab[*,1,1,1]),' +- ',stddev(pvalue_tab[*,1,1,1])
  tsigm = computeEquivSigma(reform(pvalue_tab[*,1,1,0]))
  print,'Mean corrected sigma (with ISW): ',mean(tsigm),' +- ',stddev(tsigm)
  tsigm = computeEquivSigma(reform(pvalue_tab[*,1,1,1]))
  print,'Mean corrected sigma (no ISW): ',mean(tsigm),' +- ',stddev(tsigm)
 
  ;; Correlation tests
  print,'---- Correlation tests (spherical harmonics) ----'

   print,'With complete data (log-normal)'
  idxfn = where(pvalue_tab[*,2,0,0] gt pthres, countfn )
  tfn   = double(countfn) / double(MC) 
  idxfp = where(pvalue_tab[*,2,0,1] lt pthres, countfp )
  tfp   = double(countfp) / double(MC) 
  print,'False positive: ',tfp,' False negative: ',tfn
  print,'Mean sigma (with ISW): ',mean(score_tab[*,2,0,0]),' +- ',stddev(score_tab[*,2,0,0])
  print,'Mean sigma (no ISW): ',mean(score_tab[*,2,0,1]),' +- ',stddev(score_tab[*,2,0,1])
  print,'Mean p-value (with ISW): ',mean(pvalue_tab[*,2,0,0]),' +- ',stddev(pvalue_tab[*,2,0,0])
  print,'Mean p-value (no ISW): ',mean(pvalue_tab[*,2,0,1]),' +- ',stddev(pvalue_tab[*,2,0,1])
  tsigm = computeEquivSigma(reform(pvalue_tab[*,2,0,0]))
  print,'Mean corrected sigma (with ISW): ',mean(tsigm),' +- ',stddev(tsigm)
  tsigm = computeEquivSigma(reform(pvalue_tab[*,2,0,1]))
  print,'Mean corrected sigma (no ISW): ',mean(tsigm),' +- ',stddev(tsigm)

  print,'With inpainting (log-normal)'
  idxfn = where(pvalue_tab[*,2,1,0] gt pthres, countfn )
  tfn   = double(countfn) / double(MC) 
  idxfp = where(pvalue_tab[*,2,1,1] lt pthres, countfp )
  tfp   = double(countfp) / double(MC) 
  print,'False positive: ',tfp,' False negative: ',tfn
  print,'Mean sigma (with ISW): ',mean(score_tab[*,2,1,0]),' +- ',stddev(score_tab[*,2,1,0])
  print,'Mean sigma (no ISW): ',mean(score_tab[*,2,1,1]),' +- ',stddev(score_tab[*,2,1,1])
  print,'Mean p-value (with ISW): ',mean(pvalue_tab[*,2,1,0]),' +- ',stddev(pvalue_tab[*,2,1,0])
  print,'Mean p-value (no ISW): ',mean(pvalue_tab[*,2,1,1]),' +- ',stddev(pvalue_tab[*,2,1,1])
  tsigm = computeEquivSigma(reform(pvalue_tab[*,2,1,0]))
  print,'Mean corrected sigma (with ISW): ',mean(tsigm),' +- ',stddev(tsigm)
  tsigm = computeEquivSigma(reform(pvalue_tab[*,2,1,1]))
  print,'Mean corrected sigma (no ISW): ',mean(tsigm),' +- ',stddev(tsigm)
 
  save,/variables,filename=savefinal
  stop

end
