;
; Perform the Saclay Method for ISW detection on input data
; This method is composed of three steps, each involving a different kind of tests
; Everything is performed inside the spherical harmonic domain.
;
; AUTHOR: Francois-Xavier Dupe
; DATE: 01/09/2010
; REVISION:
;

;
; isw_saclayMethod: Perform the Saclay method on input data
; INPUT:
;    cmbobs:        the observed CMB
;    galobs:        the observed matter over-density (generally from galaxy over-density) or the galaxy survey (with its mask)
;    iswspec:       the expected cross-power spectra between matter and CMB (in spherical harmonics)
;    cmbspec:       the expected primordial CMB power spectra
;    galspec:       the expected matter power spectra
;    cmbmask:       the mask for CMB data (for the inpainting step)
;    galmask:       the mask for matter density data (for the inpainting step)
;    mbias:         bias between matter / galaxy densities (for inpainting step)
;    lmin:          the lower multipole inside spherical harmonics
;    lmax:          the upper multipole inside spherical harmonics
;    inpainting:    toggle for performing inpainting or not
;    s1iter:        the number of bootstrap iterations for the p-value estimation inside the first step (correlation detection)
;    s2iter:        the number of bootstrap iterations for the p-value estimation inside the second step (amplitude estimation)
;    s3iter:        the number of Monte-Carlo iterations for the p-value estimation inside the third step (model selection)
;
pro isw_saclayMethod,cmbobs,galobs,iswspec,cmbspec=cmbspec,galspec=galspec,cmbmask=cmbmask,galmask=galmask,mbias=mbias,lmin=lmin,lmax=lmax,$
                     inpainting=inpainting,s1iter=s1iter,s1nestiter=sinestiter,s2iter=s2iter,s3iter=s3iter

  if not keyword_set(lmin)  then lmin = 3
  if not keyword_set(lmax)  then lmax = 100
  if not keyword_set(mbias) then mbias = 1.0d

  if not keyword_set(s1iter)     then s1iter = 1000
  if not keyword_set(s1nestiter) then s1nestiter = 1001
  if not keyword_set(s2iter)     then s2iter = 10000
  if not keyword_set(s3iter)     then s3iter = 10000

  ;; Perform CMB inpainting if the mask is given
  if keyword_set(cmbmask) and keyword_set(inpainting) then begin
     cmbinpaint = mrs_alm_inpainting(cmbobs*cmbmask,mask=cmbmask,opt='-i40 -v')
     cmbobs = cmbinpaint
  endif

  ;; Perform matter over-density inpainting from galaxy survey if the mask is give
  if keyword_set(galmask) and keyword_set(inpainting) then begin
     galinpaint = mrs_alm_inpainting(galobs*galmask,mask=galmask,opt='-v -i40 -T -C -p')
     galmean = mean(galinpaint)
     galobs = (galintpaint - galmean)/galmean / mbias
  endif

  ;; First step: correlation detection
  t1 = isw_corrTest(cmbobs,galobs,method=0,lmin=lmin,lmax=lmax,bootstrap=s1iter,btstpvar=1001,bootmet=1,/noverb)
  print,'First step: correlation detection'
  print,'p-value =',t1.pvalue
  print,'sigma = ',inverf(1.0d - t1.pvalue)*sqrt(2.0d)

  ;; Second step: signal detection / amplitude estimation
  t2 = isw_estAmpl(cmbobs,galobs,iswspec=iswspec,cmbspec=cmbspec,galspec=galspec,lmin=lmin,lmax=lmax,bootstrap=s2iter,btspvar=0,/noverb,/usemc)
  print,'Second step: amplitude estimation (aka signal detection)'
  print,'p-value =',t2.pvalue
  print,'sigma = ',inverf(1.0d - t2.pvalue)*sqrt(2.0d)

  ;; Third and final step: model selection
  t3 = isw_testModel(cmbobs,galobs,iswspec=iswspec,cmbspec=cmbspec,galspec=galspec,lmin=lmin,lmax=lmax,bootstrap=s3iter,/noverb)
  print,'Third step: model selection'
  print,'p-value =',t3.pvalue
  print,'sigma = ',inverf(1.0d - t3.pvalue)*sqrt(2.0d)

end
