;
; GLR test between two models (one with ISW theoritical signal, one without) using data from a simulation
; Use a single parametric bootstrap in order to estimate the distribution of the test (seems to be an exact test)
;
; AUTHOR: Francois-Xavier Dupe
; DATE: 04/2010
; REVISION: 20/05/10 New version including a single bootstrap
;           27/08/10 Rewrite the test in order to optimize it for IDL
;

;
; isw_testModel: Conduct the GLR test on data
; INPUT:   icmbibs:   the CMB observation
;          igalobs:   the matter density field
;          cmbspec:   the power spectra of the primordial CMB
;          galspec:   the power spectra of the matter density
;          iswspec:   the cross power spectra between temperature and matter density
;          lmin:      the lower pole
;          lmax:      the upper pole
;          bootstrap: the number of bootstrap iteration (here a Monte Carlo)
;          noverb:    set to true for disabling information printing
;          nosmooth:  set to true for disabling smoothing during p-value estimation
; OUTPUT:  a structure containing information on the result of the test
;
function isw_testModel,icmbobs,igalobs,cmbspec=cmbspec,galspec=galspec,iswspec=iswspec,lmin=lmin,lmax=lmax,bootstrap=bootstrap,noverb=noverb,$
                      nosmooth=nosmooth

  if not keyword_set(bootstrap) then bootstrap=100

  ;; 1 - Create vector for the data manipulation

  mrs_almtrans, icmbobs, icmbalm, /tab, /complex, lmax=lmax
  mrs_almtrans, igalobs, ogalm, /tab, /complex, lmax=lmax

  almcount = 0ul
  for l=lmin,lmax do almcount = almcount+2*l+1ul

  cmbalm = dcomplexarr(almcount)
  iswalm = dcomplexarr(almcount)
  galalm = dcomplexarr(almcount)
  cmbvar = dblarr(almcount)
  galvar = dblarr(almcount)
  iswvar = dblarr(almcount)

  ;; 2 - Do some estimations from perturbed observation
  ;; Use the theoritical spectra or estimate them
  
  cgg = dblarr(lmax+1)
  cgt = dblarr(lmax+1)
  cnn = dblarr(lmax+1)
   
  for l=0,lmax do begin
     
     vcgg = dblarr(2*l+1)
     vcgt = dblarr(2*l+1)
     vcnn = dblarr(2*l+1)
     
     for m=0,l do begin 
        vcgg[m] = abs(ogalm.alm[l,m])^2
        vcgt[m] = real_part(ogalm.alm[l,m]*conj(icmbalm.alm[l,m]))
        vcnn[m] = abs(icmbalm.alm[l,m])^2
     endfor
     for m=1,l do begin 
        vcgg[l+m] = abs(ogalm.alm[l,m])^2
        vcgt[l+m] = real_part(ogalm.alm[l,m]*conj(icmbalm.alm[l,m]))
        vcnn[l+m] = abs(icmbalm.alm[l,m])^2
     endfor
     
     cgg[l] = mean(vcgg)
     cgt[l] = mean(vcgt)
     cnn[l] = mean(vcnn)
     
  endfor
  
  if keyword_set(cmbspec) then cnn=cmbspec
  if keyword_set(galspec) then cgg=galspec
  if keyword_set(iswspec) then cgt=iswspec

  iiswalm = ogalm
  for l=0,lmin-1  do iiswalm.alm[l,*] *= 0.0d
  for l=lmin,lmax do iiswalm.alm[l,*] *= cgt[l]/cgg[l] ; Apply the Fc filter

  ;; 3 - Put data into the big vectors
  iter = 0ul
  for l=lmin,lmax do begin
     for m=0,l do begin
        cmbalm[iter] = icmbalm.alm[l,m]
        iswalm[iter] = iiswalm.alm[l,m]
        galalm[iter] = ogalm.alm[l,m]
        cmbvar[iter] = cnn[l]
        galvar[iter] = cgg[l]
        iswvar[iter] = cgt[l]
        iter = iter + 1ul
     endfor
     for m=1,l do begin
        cmbalm[iter] = icmbalm.alm[l,m]
        iswalm[iter] = iiswalm.alm[l,m]
        galalm[iter] = ogalm.alm[l,m]
        cmbvar[iter] = cnn[l]
        galvar[iter] = cgg[l]
        iswvar[iter] = cgt[l]
        iter = iter + 1ul
     endfor
  endfor

  ;; 3 - Test the current model with the data

  xglr = real_part(transpose(conj(cmbalm)) # (cmbalm / cmbvar)) - $
         real_part(transpose(conj(cmbalm - iswalm)) # ((cmbalm - iswalm) / cmbvar))
  xglr = xglr[0]
 
  ;; 4 - Use Bootstrap in order to estimate the distribution of the test under H0

  nres  = randomn(seed,/normal) ; init random
  brslt = dblarr(bootstrap)
  for B=0ul,bootstrap-1 do begin

     ;; New field under the null hypothesis
     cmbsignal = dcomplexarr(almcount)
     iternw = 0ul
     for l=lmin,lmax do begin
        
        rv0  = randomn(seed,1,/normal,/double)*sqrt(cmbspec[l])
        rvar = randomn(seed,l,/normal,/double)*sqrt(cmbspec[l]/2.0d)
        ivar = randomn(seed,l,/normal,/double)*sqrt(cmbspec[l]/2.0d)
        
        cmbsignal[iternw]            = rv0
        cmbsignal[iternw+1:iternw+l] = dcomplex(rvar,ivar)
        cmbsignal[iternw+l+1ul:iternw+2ul*l] = cmbsignal[iternw+1:iternw+l]
        
        iternw = iternw + 2 * l + 1ul
        
     endfor

     newxglr = real_part(transpose(conj(cmbsignal)) # (cmbsignal / cmbvar)) - $
               real_part(transpose(conj(cmbsignal - iswalm)) # ((cmbsignal - iswalm) / cmbvar))
     brslt[B] = newxglr[0]

  endfor

  ;; 5 - Compute the p-value

  if keyword_set(nosmooth) then begin
     
     aix = where(brslt le xglr, count) ; Brutal (exact estimation of the p-value)
     
  endif else begin
     
     count     = 0.0d
     ;; bandwidth = 1.3167d * double(bootstrap)^(-4.0d / 9.0d) 
     bandwidth = 1.0d * double(bootstrap)^(-4.0d / 9.0d) 
     for iter=0ul,bootstrap-1 do count = count + gaussint((xglr - brslt[iter])/bandwidth)
     
  endelse
  
  pvalue = 1.0d - double(count)/double(bootstrap)
  rscore = inverf(1.0d - pvalue)*sqrt(2.0d)

  if not keyword_set(noverb) then print,'GLR test: ',xglr     
  if not keyword_set(noverb) then print,'p-value: ',pvalue
  if not keyword_set(noverb) then print,'sigma: ',rscore

  res = { testval: xglr, score: xglr, pvalue: pvalue, real_score: rscore }

  return,res

end
