;+
; NAME:
;        mrs_gal_inpainting
;
; PURPOSE:
;       Denoising and inpainting algorithm for galaxy density using Poisson data fidelity, log-normal/normal prior and
;        sparse prior inside the spherical harmonics. A primal-dual algorithm is used for the minimization.
;
; CALLING:
;                   ImaInp = mrs_gal_inpainting(GalaxiesIma,  Mask,  nbIter=nbIter,   nbEM=nbEM, tau=tau,  lmax=lmax, 
;                                                               verb=verb,  normal=normal, pspec=pspec, nbtex=nbtex, MatMask=MatMask )';
; INPUTS:
;     Imag -- IDL array of healpix map: Input image to be inpainted 
;     Mask -- IDL array of healpix map: Input Mask of missing data (Mask[k] =0, if pixel k is missing) 
;    
; OUTPUTS:
;     ImaInp -- -- IDL array of healpix map: Ouput inpainted image  
;
; OPTIONAL INPUT KEYWORDS:
;     nbte -- int  = number of iterations for the texture synthesis
;     nbem -- int  = number of iterations for the EM algorithm. If nbem = 0, then no inpainting is applied and it is just a 
;                                                            denoising algorithm using the maximum likelihood method (Kitaura method).
;     tau -- double = Theoretically, it should be the inverse of the maximum value of the power spectrum,
;
; EXTERNAL CALLS:
;       mrs_powspec.pro
;       mrs_deconv_powspec.pro
;
; EXAMPLE:
;       Inpaint+Poisson denoising of an image  
;                ImaInp = mrs_gal_inpainting(GalaxiesIma,  Mask)
;         
; HISTORY:
;	Written: Francois-Xavier Dupe, 2011  
;	March, 2011 File creation
;--------------------------------------------------------------------------------------------------------

 function fx_galaxyDenoising,galobs,mask=mask,nbIter=nbIter,theta=theta,lmax=lmax,verb=verb,gmean=gmean,pspec=pspec,sigma=sigma,tau=tau,$
                            nbInner=nbInner,support=support,NbrScale=NbrScale

  if not keyword_set(nbIter)    then nbIter   = 100
  if not keyword_set(nbInner)   then nbInner  = 100
  if not keyword_set(mask)      then mask     = 1.0d
  if not keyword_set(theta)     then theta    = 1.0d
  if not keyword_set(sigma)     then sigma    = 1.0d
  if not keyword_set(tau)       then tau      = 0.1d
  if not keyword_set(support)   then support  = 0.0d ;; Force nothing
  if not keyword_set(NbrScale)  then NbrScale = 6

  npix = n_elements(galobs)

  ;; Force things to be double
  galobs = double(galobs)
  theta  = double(theta)
  gmean  = double(gmean)
  pspec  = double(pspec)

  ;; Convert the power spectra for log-normal
  cggln = alog(1.0d + pspec)
  cgglndiv = cggln

  cggln[0] = 0.0d
  cgglndiv[0] = 1.0d
  
  initobs = (double(galobs) - gmean) / gmean
  idx = where(initobs le -1.0d, count)
  if count gt 0 then initobs[idx] = -1.0

  ;; Compute the target of the multi-resolution support
  if keyword_set(support) then begin
     dobs = initobs
     idx = where(dobs le -1.0d, count)
     if count gt 0 then dobs[idx] = -0.99d ;; Default value
     mrs_wttrans,alog(1.0d + dobs),dcoeff,NbrScale=NbrScale
  endif

  ;; Initialization of the variables
  
  xbar = initobs*0.0d
  mrs_almtrans,initobs*0.0d,zalm,/tab,/complex,lmax=lmax
  xv   = xbar
  yv   = xbar
  tv   = xbar
  if keyword_set(support) then wv = dcoeff else wv = 0.0d
  zv   = zalm

  ypix = xbar

  if keyword_set(support) then wv.coef = wv.coef * 0.0d

  ;; Size of the vector
  almcount = 0ul
  mu       = dcomplexarr(lmax+1,lmax+1)
  for l = 0,lmax do begin
     for m = 0,l do begin
        almcount = almcount + 1ul
        mu[l,m]  = dcomplex(-0.5d/sqrt(2.0d),-0.5d/sqrt(2.0d)) ;; The mean of the Gaussian field
     endfor
  endfor
  mu[0,*] = mu[0,*] * 0.0d

  ;; Perform a generalize Douglas-Rachford scheme for computing the minimum
  for iter = 0,nbIter-1 do begin

     if keyword_set(verb) then print,'Iteration ',iter

     ;; 1 - Prepare things for the dual projections

     if keyword_set(support) then begin
        mrs_wttrans,xbar,xcoeff,NbrScale=NbrScale
        cv      = xcoeff
        cv.coef = wv.coef + sigma * support * xcoeff.coef
     endif

     mrs_almtrans,xbar,xalm,/tab,/complex,lmax=lmax
     bv = xalm
     
     for l = 0,lmax do if cggln[l] gt 0.0d then bv.alm[l,*] = xalm.alm[l,*] / sqrt(cgglndiv[l])
     bv.alm = zv.alm + sigma * bv.alm

     yv = ypix + sigma * mask * xbar

     ;; 2 - The multi-resolution support

     if keyword_set(support) then wv.coef = cv.coef - sigma * support * dcoeff.coef

     ;; 3 - The proximal projector associated to the exponential
     
     wz  = gmean / sigma * exp(yv / sigma - (1.0d - double(galobs)) / sigma)
     wwj = wz * 0.0d ;; Init

     for inner = 0,nbInner-1 do $
        wwj = wwj - (wwj*exp(wwj) - wz) / (exp(wwj)*(wwj + 1.0d) - (wwj + 2.0d)*(wwj*exp(wwj) - wz)/(2.0d * wwj + 2.0d))
     
     targ = alog(wwj/(gmean/sigma))
     
     ypix = yv - sigma * targ

     ;; 4 - The gaussian data fidelity (here the log of the field)
     
     ;; tmp = bv
     ;; epsilon = sqrt(total(abs(bv.alm/sigma - mu)^2.0d))

     ;; ;; print,'epsilon = ',epsilon
     ;; ;; print,'beta * almcount = ',beta*sqrt(almcount)

     ;; if epsilon le beta * sqrt(almcount) then tmp.alm = bv.alm/sigma - mu else tmp.alm = (bv.alm / sigma - mu) / epsilon * beta * sqrt(almcount)
     ;; zv.alm = bv.alm - sigma * ( mu + tmp.alm )

     zv.alm = bv.alm - sigma / (1.0d + 1.0d / sigma) * ( mu / sigma + bv.alm / sigma )

     ;; 5 - Prepare things for the primal projection
     
     if keyword_set(support) then begin
        wtmp = wv
        wtmp.coef = wv.coef * support
        mrs_wtrec,wtmp,wpix
     endif

     bv = zv
     for l = 0,lmax do bv.alm[l,*] = zv.alm[l,*] * sqrt(cggln[l])
     mrs_almrec,bv,zpix
     
     if keyword_set(support) then $
        tv = xv - tau * (wpix + mask * ypix + zpix) $
     else $
        tv = xv - tau * (mask * ypix + zpix)

     ;; 6 - Final update

     xbar = tv + theta * (tv - xv)
     xv   = tv

     if keyword_set(verb) then begin

        mrs_almtrans,xv,xvalm,/tab,/complex,lmax=lmax
        kalm = xvalm
        for l = 0,lmax do if cgglndiv[l] gt 0.0d then  kalm.alm[l,*] = xvalm.alm[l,*] / sqrt(cgglndiv[l])

        res = total(gmean * exp(xv)) + total(transpose(1.0d - double(galobs)) # xv)
        res = res +  1.0d / 2.0d * total(abs(kalm.alm[1:lmax,*] - dcomplex(-0.5d,-0.5d))^2.0d)

        print,'res = ', res

     endif

  endfor

  ;; Contruct the final estimate
  est = exp(xv) - 1.0d

  return,est

end

;==================================================================================

;
; fx_galaxyInpaint: galaxy overdensity inpainting and denoising function
;
;     fx_galaxyInpaint,galobs,mask=mask,nbIter=nbIter,
function fx_galaxyInpaint,galobs,mask=mask,nbIter=nbIter,nbInner=nbInner,nbEM=nbEM,tau=tau,theta=theta,lmax=lmax,verb=verb,$
                          normal=normal,pspec=pspec,gmean=gmean,resverb=resverb,emnormal=emnormal,nbMI=nbMI,$
                          nbtex=nbtex,updatespec=updatespec,eststop=eststop,epsilon=epsilon,support=support,NbrScale=NbrScale,sigma=sigma,$
                          coef_seuil=coef_seuil,nbPSem=nbPSem

  if not keyword_set(nbtex)  then nbtex  = 5
  if not keyword_set(nbPSem) then nbPSem = 1
  
   galindex = where(mask ne 0.0d)
   masindex = where(mask eq 0.0d)
   npix     = n_elements(galobs)

   ;; Correctly estimate the mean count
     if not keyword_set(gmean) then begin
        galmean = mean(galobs[galindex])
     endif else $
        galmean = gmean

     if keyword_set(verb) then print,'GMean = ',galmean

  if not keyword_set(nbEM) then begin
     
     ;; Compute the new support
     if keyword_set(support) then begin
        print,'Compute support'
        mrs_msvsts_IUWT_hypothesis_testing,double(galobs),imgvst,support,imgrec,NbrScale=NbrScale,coef_seuil=coef_seuil
        
        ;; Complete support by adding the last scale
        tsupport        = dblarr(n_elements(galobs),NbrScale)
        for i = 0,NbrScale-2 do tsupport[*,i] = support[*,i]
        tsupport[*,NbrScale-1] = double(galobs)*0.0d + 1.0d
     endif

     kspec=pspec ;; Change the name
     for emps = 0,nbPSem-1 do begin
        
        if keyword_set(verb) then print,'EM power spectra ',emps
        
        est = fx_galaxyDenoising(double(galobs),mask=mask,nbIter=nbIter,nbInner=nbInnner,theta=theta,lmax=lmax,verb=verb,$
                                 gmean=galmean,pspec=kspec,support=tsupport,NbrScale=NbrScale,sigma=sigma,tau=tau)

        kspec = mrs_powspec(est,/nonorm,lmax=lmax) ;; Renew the power spectra estimate
        
     endfor

     return,   (est +1.)*galmean

  endif else begin

     if not keyword_set(emmeth) then emmeth = 1 ;; By default use the data augmentation method
     ;; estdens = (galobs - gmean) / gmean ;; Current estimation of the density field

     ;; Use a Data augmentation process in order to infer missing data and its statistics
     
     ;; Use EM mechanism in order to build an inpaint version of the galaxy density field
     if not keyword_set(nbMI)  then nbMI  = 1
     nest     = dblarr(npix,nbMI)

 
   
     ;; Initialize the algorithm with a rough estimation of the observed density
     for imi = 0,nbMI-1 do nest[*,imi] = (galobs - galmean) / galmean

     if not keyword_set(pspec) then           galpspec = mrs_powspec(reform(nest[*,0]),/nonorm,lmax=lmax) $
     else                                     galpspec = pspec
     if n_elements(galpspec) gt (lmax+1) then galpspec = galpspec[0:lmax] ;; Avoid some strange things

     aix = where(galpspec lt 0.0d, count)
     if count gt 0 then galpspec[aix] = 0.0d

     for iter = 0,nbEM-1 do begin

        if keyword_set(verb) then print,'EM ITERATION ',iter
        est         = dblarr(npix,nbMI)
        galobs_comp = dblarr(npix,nbMI)

        for imi = 0,nbMI-1 do begin
           
           if keyword_set(verb) then print,'Imputation ',imi

           estdens = reform(nest[*,imi])

           mrs_almtrans,estdens,ealm,/tab,/complex,lmax=lmax
           mrs_almrec,ealm,estdens ;; Reduce the estimation to lmax

           ;; Be careful about the minimum value
           idx = where(estdens le -1.0d, count)
           if count gt 0 then estdens[idx] = -0.99d
           
           ;; 1- Build a initial field using given spectra
           if not keyword_set(emnormal) then begin
              
              ;; Generate a new galaxy field using log-normal distribution
              for l = 0,lmax do begin
                 cggln = alog(1.0d + galpspec[l])
                 
                 for m = 0,l do begin
                    
                    gfx = randomn(seed, /normal, /double)
                    gfy = randomn(seed, /normal, /double)
                    
                    ealm.alm[l,m] = dcomplex(sqrt(cggln/2.0d)*gfx - cggln/2.0d,sqrt(cggln/2.0d)*gfy - cggln/2.0d)
                 endfor
              endfor
              
              mrs_almrec,ealm,galln
              galln = exp(galln) - 1.0d
              
           endif else begin
              
              ;; Generate a new galaxy field using normal distribution
              for l = 0,lmax do begin
                 for m = 0,l do begin
                    gfx = randomn(seed, /normal, /double)
                    gfy = randomn(seed, /normal, /double)
                    
                    ealm.alm[l,m] = dcomplex(sqrt(galpspec[l]/2.0d)*gfx,sqrt(galpspec[l]/2.0d)*gfy)
                 endfor
              endfor
              mrs_almrec,ealm,galln

           endelse
           
           ;; 2- Use texture generation method in order to build realistic data for the missing part
           currest = mask * estdens + (1.0d - mask) * galln

           for ittex = 0,nbtex-1 do begin

              ;; 2.0 Protection because of the logarithm
              idx = where(currest le -1.0d, count)
              if count gt 0 then currest[idx] = -0.99d

              mrs_almtrans,alog(1.0d + currest),curalm,/tab,/complex,lmax=lmax
              currspec = mrs_powspec(alog(1.0d + currest),lmax=lmax,/nonorm)

              aix = where(currspec lt 0.0d, count)
              if count gt 0 then currspec[aix] = 0.0d
              
              ;; 2.1 Impose the given power spectra
              for l = 0,lmax do begin
                 cggln = alog(1.0d + galpspec[l])
                 if currspec[l] gt 0 then curalm.alm[l,*] = curalm.alm[l,*] * sqrt(cggln / currspec[l])
              endfor
              
              ;; 2.2 Impose the null mean
              mrs_almrec,curalm,currest
              currest = exp(currest) - 1.0d
              currest = currest - mean(currest)                 

              ;; 2.3 Impose the minimum value
              idx = where(currest le -1.0d, count)
              if count gt 0 then currest[idx] = -1.0d
              
              ;; 2.4 Update the current estimate
              currest = mask * estdens + (1.0d - mask) * currest

           endfor
           
           currest = (1.0d + currest) * galmean

           ;; 3- Add noise to the "new" data
           galobs_new = double(galobs)
           for i = 0ul,n_elements(masindex)-1 do if currest[masindex[i]] gt 0 then galobs_new[masindex[i]] = randomn(seed,poisson=currest[masindex[i]])

           if keyword_set(eststop) then stop

           galobs_comp[*,imi] = galobs_new ;; Add to the vector of complete observations

           ;; Compute the new support
           if keyword_set(support) then begin
              print,'Compute support'
              mrs_msvsts_IUWT_hypothesis_testing,galobs_new*mask,imgvst,support,imgrec,NbrScale=NbrScale,coef_seuil=coef_seuil
              
              ;; Complete support by adding the last scale
              tsupport = dblarr(n_elements(galobs_new),NbrScale)
              for i = 0,NbrScale-2 do tsupport[*,i] = support[*,i] * double(mask)
              tsupport[*,NbrScale-1] = double(mask)*1.0d ;; + 1.0d
           endif
           
           ;; 4- Estimate the density map with the power spectra
           kpspec = galpspec
           for emps = 0,nbPSem-1 do begin
              
              if keyword_set(verb) then print,'EM PS estim ',emps
              
              est[*,imi] = fx_galaxyDenoising(double(galobs_new),nbIter=nbIter,nbInner=nbInnner,theta=theta,lmax=lmax,verb=verb,$
                                              gmean=galmean,pspec=kpspec,support=tsupport,NbrScale=NbrScale,sigma=sigma,tau=tau)

              ;; Enforce the null mean
              est[*,imi] = reform(est[*,imi]) - mean(reform(est[*,imi]))

              kpspec = mrs_powspec(reform(est[*,imi]),/nonorm,lmax=lmax) ;; Renew the power spectra estimate
              
           endfor
        endfor

        ;; 5 - Update the parameters from the different imputations
        ;; 5.1 - The new estimate
        nest = est 

        ;; 5.2 - The new power spectra
        if keyword_set(updatespec) then begin
           old_pspec = galpspec
           galpspec = galpspec * 0.0d
           for imi = 0,nbMI-1 do begin
              gspec = mrs_powspec(reform(est[*,imi]),/nonorm,lmax=lmax)
              aix = where(gspec lt 0.0d, count)
              if count gt 0 then gspec[aix] = 0.0d
              galpspec = galpspec + gspec
           endfor
           galpspec = galpspec / nbMI
        endif else begin
           old_pspec = galpspec
           for imi = 0,nbMI-1 do old_pspec = old_pspec + mrs_powspec(reform(est[*,imi]),/nonorm,lmax=lmax)
           old_pspec = old_pspec / nbMI
        endelse

        if keyword_set(verb) then print,'Convergence criterium = ',total(abs(old_pspec - galpspec)) / double(lmax+1)

     endfor
     
     return,   (est +1.)*galmean    ;; Return all the estimates (could be useful for MC schemes)

  endelse
  
  return,galobs
  
end


;==================================================================================

function mrs_gal_inpainting,  GalaxiesIma,  Mask,  nbIter=nbIter,   nbEM=nbEM, tau=tau,  lmax=lmax, verb=verb,  Pspec=Pspec, nbtex=nbtex, MatMask=MatMask, mu=mu, DeconvIter=DeconvIter
                          
if N_PARAMS() LT 2 then begin 
        print, 'CALLING SEQUENCE: ImaInp = mrs_gal_inpainting(GalaxiesIma,  Mask,  nbIter=nbIter,   nbEM=nbEM, tau=tau,  lmax=lmax, verb=verb,  pspec=pspec, nbtex=nbtex, MatMask=MatMask )'
	    ima_inp =-1
        goto, DONE
        end

if not keyword_set(nbtex)  then nbtex  = 5
if not keyword_set(nbem) then nbem = 5
 if not keyword_set(nbIter)  then nbIter  = 500
 if not keyword_set(tau)  then tau=0.0002d
 if not keyword_set(DeconvIter)  then DeconvIter  = 20

Ima = GalaxiesIma * Mask
ind = where(Mask NE 0)
MeanIma = mean(Ima[ind])
Ima[ind] = (Ima[ind] - MeanIma) / MeanIma

if not keyword_set(pspec) then begin
   PowSpecMaskedData = mrs_powspec(Ima, lmax=lmax)
   Pspec  = mrs_deconv_powspec(PowSpecMaskedData,  Mask,  MatMask=MatMask,  Niter=DeconvIter,  lmax=lmax, mu=mu)
    ; Pspec =  mrs_master_powspec(Ima,  Mask, lmax=lmax)
end
if not keyword_set(lmax) then begin
   vs = size(Pspec)
   lmax = vs[1]-1
end

ImaM = GalaxiesIma * Mask
ima_inp  = fx_galaxyInpaint(ImaM,  mask=Mask, pspec=Pspec, nbIter=nbIter, lmax=lmax, verb=verb, tau=tau,  nbem=nbem, nbtex=nbtex)

DONE:
 
  RETURN, ima_inp
end

;==================================================================================

pro test_gal_inp
DIR = getenv("ISAP") + '/data/ISW_DATA/2MASS/'
data32 = rims(DIR+'2mass_nside32.fits')
mask32 = rims(DIR+'2mass_mask_nside32.fits')
ima = mrs_gal_inpainting(data32,  mask32,  pspec=pspec, MatMask=MatMask)
end

;==================================================================================


