;+
; NAME:
;        mrs_deconv_powspec
;
; PURPOSE:
;   Computes the power spectrum of a map with missing data, 
;   using the HEALPix representation (nested data
;   representation by default). The MASTER method is used, but using an iterative technique instead
;   of a brute force matrix inversion. If a noise power spectrum is given, it is removed from the solution.
;   The noise power spectrum is assumed to be unbiased, i.e. free of missing area.
;  
; CALLING:
;     P = mrs_deconv_powspec( PowSpecMaskedData,  Mask,  MatMask=MatMask,  Niter=Niter,  NoisePS=NoisePS, lmax=lmax)
;
; INPUTS:
;     PowSpecMaskedData -- IDL array : Power spectrum of the masked data   
;     Mask -- IDL array of healpix map: Input Mask of missing data (Mask[k] =0, if pixel k is missing) 
;     
; OUTPUTS:
;     P -- 1D IDL fltarr:  Power Spectrum corrected from the mask,
;
; OPTIONAL INPUT KEYWORDS:
;     PSCMB_Mask -- IDL array:   power  spectrum of the mask
;     Niter -- int: Number of iteration. Default is 20.
;     NoisePS -- IDL 1D array: Noise power spectrum. Default is none.
;
; INPUT/OUTPUT KEYWORDS:
;      MatMask -- IDL 2D array: Master matrix related to the mask (see mrs_matmask.pro)
;     
; EXTERNAL CALLS:
;       mrs_matmask.pro
;
; EXAMPLE:
;       Compute the power spectrum of an image. 
;               P = mrs_master_powspec(Imag, Mask) 
;         
; HISTORY:
;	Written: Jean-Luc Starck 
;	October, 2011 File creation
;--------------------------------------------------------------------------------------------------------

function mrs_deconv_powspec,  PowSpecMaskedData,  Mask,  MatMask=MatMask, PRes, Niter=Niter, TrueCl=TrueCL, NoisePS=NoisePS, lmax=lmax, mu=mu, zeromonodip=zeromonodip, verb=verb, fsky=fsky, inv=inv, ImatMask=IMatmask, IdealBeam=IdealBeam

if N_PARAMS() LT 1 then begin 
        print, 'CALLING SEQUENCE: p = mrs_deconv_powspec( PowSpecMaskedData,  Mask,  MatMask=MatMask, niter=niter, NoisePS=NoisePS, lmax=lmax)'
	    Pres =-1
        goto, DONE
        end

PDat = PowSpecMaskedData
if keyword_set(lmax) then begin
    vs = size(PowSpecMaskedData)
    Nl = vs[1] +1 
    if lmax lt Nl then PDat = Pdat[0:lmax]
 
end
if not keyword_set(NoisePS) then NoisePS=0.

; Brute force Inversion of the matrix
if keyword_set(inv) or keyword_set(IMatmask) then begin
   if not keyword_set(IMatmask) then IMatmask = invert (matmask)
   PRes = IMatmask # PowSpecMaskedData
   if keyword_set(NoisePS) then PRes = PRes - IMatmask # NoisePS
    ; Positivity constraint
    ind = where(Pres LT 0, c)
    if c GT 0 then Pres[ind] = 0
    
end else begin

if not keyword_set(niter) then niter=40
if not keyword_set(MatMask) then MatMask = mrs_matmask(Mask, lmax=lmax)
if not keyword_set(mu) then mu = 1.


if not keyword_set(fsky) and keyword_set(Mask) then Fsky = float( N_ELEMENTS(Mask) / float ( total(mask)) ) 
if not keyword_set(fsky) then Fsky=1

; print, fsky

; Firt initialisation: Fsky correction and noise subtraction
PRes = PDat*Fsky - NoisePS
;Energy = total(Pres)
;print, ", Energ = ", total(Pres)

Tmat = transpose(MatMask)
; mu = 1./ fsky^2
if keyword_set(IdealBeam) then begin
	vs = size(PDat)
	Nl = vs[1]
	LowPassFilter = dblarr(Nl)
	vs = size(IdealBeam)
	Nl1 = vs[1]
	if NL1 gt Nl then LowPassFilter = IdealBeam[0:Nl-1] $
	else LowPassFilter[0:Nl1-1] = IdealBeam
end

for i=0,Niter-1 do begin
   Resi = PDat - MatMask # ( PRes + NoisePS )
   if keyword_set(zeromonodip) then PRes[0:1] = 0
   
   if keyword_set(IdealBeam) then  PRes += mu * (Tmat # resi) * LowPassFilter  $
   else PRes += mu * (Tmat # resi) 
   
   if  keyword_set(verb) then print, i+1, ': Resi = ', sigma(Resi), ", Energ = ", total(Pres)
   ; Positivity constraint
   ind = where(Pres LT 0, c)
   if c GT 0 then Pres[ind] = 0
   ; Pres = Pres * Energy / total(Pres)
   if keyword_set(TrueCl) then begin
      plotcl, TrueCl, thick=2
      oplotcl, Pres
   end
end
end 
 
 
DONE:
 
  RETURN, Pres
end


