/******************************************************************************
**                   Copyright (C) 2009 by CEA
*******************************************************************************
**
**    UNIT
**
**    Version: 1.0
**
**    Author: Florent Sureau
**
**    Date:  09/12/09
**    
**    File:  mrs_planck_mgmca.cc
**
*******************************************************************************
**
**    DESCRIPTION  Multiscale Generalized Morphological Component Analysis for planck datas
**    ----------- 
**                 
**    Usage: mrs_planck_mgmca options cube output
**
******************************************************************************/

#include "HealpixClass.h"
#include "MRS_Sparse.h"
#include "mrs_planck_lGMCA.h"
#include <sys/time.h>
#include "openmp_support.h"
#include "omp_routines.h"

char Name_Cube_In[1024];
char Name_Beams[1024];
char Name_Weights[1024];
char Name_Struc[1024];
char Name_Out[1024];
int NbrScale2d = 5;

// type_transform  Transform=TO_MALLAT;

extern int  OptInd;
extern char *OptArg;

extern int  GetOpt(int argc, char *const*argv, char *opts);

bool Verbose=false;
bool lNested_in=true;
bool set_threads=false;

int NbrChan=0;  // Number of channels 
float KMin = 3; // Last K-Mad Thresholding
int Nside = 0;


/*********************************************************************/


static void usage(char *argv[])
{
    fprintf(OUTMAN, "Usage: %s [options] NameCubeIn Name_Beams Name_Weights Name_Struc Name_Out\n\n", argv[0]);
    fprintf(OUTMAN, "   where  :  \n");

    
    fprintf(OUTMAN, "         [NameCubeIn]\n");
    fprintf(OUTMAN, "             Name of fits file containing the input data [Npixels,Nchannel]. \n");
    fprintf(OUTMAN, "         [Name_Beams]\n");
    fprintf(OUTMAN, "             Name of fits file containing the beams [lmax,Nchannel]. \n");
    fprintf(OUTMAN, "         [Name_Weights]\n");
    fprintf(OUTMAN, "             Name of fits file containing the inversion weights for the wavelet coefficient for the band 0 [Npixels,Nscales,Nchannels]. \n");     
    fprintf(OUTMAN, "             This should be of the form \"*_Band0_*\" and the other resolution bands should be named \"*_BandY_*\". \n");     
    fprintf(OUTMAN, "         [Name_Struc]\n");
    fprintf(OUTMAN, "             Name of fits file containing the structure describing each required parameter [Nparam,NResBands]. \n"); 
    fprintf(OUTMAN, "             Each line should be of the form [Nchannels_used,RefChannel,used_Channels,Lmax].\n");
    fprintf(OUTMAN, "             For instance: [3 1 1 2 5 0 0 512] will use channel 1 as reference, process 1,2 and 5 (index in array: 0,1,4) degraded to the resolution of the ref channel, and use a Lmax of 512.\n");     
    fprintf(OUTMAN, "         [Name_Out]\n");
    fprintf(OUTMAN, "             Name of fits file for output of band 0. \n");     
    fprintf(OUTMAN, "             This should be of the form \"*_Band0_*\", \"_sameres_inverted\" will be added to this name, and the other resolution bands will be named accordingly \"*_BandY_*\". \n");     
    fprintf(OUTMAN, "   and [options] are  :  \n");
    fprintf(OUTMAN, "         [-r]\n");
    fprintf(OUTMAN, "             Flag to indicate that the input maps are in ring format, and not nested. Ouptut will however be in nested format. \n");     
    fprintf(OUTMAN, "         [-t oloop:iloop]\n");
    fprintf(OUTMAN, "             Use oloop outer threads and iloop inner threads . \n");     
    exit(-1);
}

/*********************************************************************/

/* GET COMMAND LINE ARGUMENTS */

static void transinit(int argc, char *argv[])
{
    int c;
#ifdef LARGE_BUFF
    int VMSSize=-1;
    Bool OptZ = False;
    char VMSName[1024] = "";
#endif   

    /* get options */
	while ((c = GetOpt(argc,argv,"?zrZ:t:")) != -1) 
    {
		switch (c) 
        {  
        	case 't':
	        	if (sscanf(OptArg,"%d:%d",&outer_loop_threads,&inner_loop_threads) < 1)
				{
		   			fprintf(OUTMAN, "Error: syntaxe is Size:Directory ... \n");
		   			exit(-1);
				}
	        	set_threads=true;
	            break;
	  		case 'r':
	        	lNested_in = false;
				break;	  		 					 				
#ifdef LARGE_BUFF
			case 'z':
	        	if (OptZ == True)
				{
					fprintf(OUTMAN, "Error: Z option already set...\n");
					exit(-1);
                }
				OptZ = True;
				break;
				
            case 'Z':
	        	if (sscanf(OptArg,"%d:%s",&VMSSize, VMSName) < 1)
				{
		   			fprintf(OUTMAN, "Error: syntaxe is Size:Directory ... \n");
		   			exit(-1);
				}
	        	
	        	if (OptZ == True)
				{
					fprintf(OUTMAN, "Error: z option already set...\n");
					exit(-1);
                }
				OptZ = True;
                break;
                
#endif
			case '?':
				usage(argv);
		}
	}
    if (OptInd < argc) strcpy(Name_Cube_In, argv[OptInd++]);
    else usage(argv);
    if (OptInd < argc) strcpy(Name_Beams, argv[OptInd++]);
    else usage(argv);
    if (OptInd < argc) strcpy(Name_Weights, argv[OptInd++]);
    else usage(argv);
    if (OptInd < argc) strcpy(Name_Struc, argv[OptInd++]);
    else usage(argv);
    if (OptInd < argc) strcpy(Name_Out, argv[OptInd++]);
    else usage(argv);

	/* make sure there are not too many parameters */
	if (OptInd < argc) {
		fprintf(OUTMAN, "Error: too many parameters: %s ...\n", argv[OptInd]);
		exit(-1);
	} 
#ifdef LARGE_BUFF
    if (OptZ == True) vms_init(VMSSize, VMSName, Verbose);
#endif
}

/*********************************************************************/
  
int main(int argc, char *argv[]) 
{
	fltarray Dat_file_out;
    /* Get command line arguments, open input file(s) if necessary */
	fitsstruct Header;
	char Cmd[512];
	struct timeval start,end,diff;

	Cmd[0] = '\0';
	for (int k =0; k < argc; k++) sprintf(Cmd, "%s %s", Cmd, argv[k]);
	   
    transinit(argc, argv);
	gettimeofday(&start, (struct timezone *) NULL);

	lGMCA_inv Testinv;
		
    #ifdef _OPENMP
    	if(set_threads == false) auto_set_inner_outer_threads(12,0,0); //Auto compute the nb procs and inner max; only specify that outer loops should be less than the number of faces (12) 
    	else set_nb_threads(outer_loop_threads,inner_loop_threads);
    #endif
    openmp_params();
  
    Testinv.Timer=true;
    Testinv.Set_Nested_in(lNested_in);
    Testinv.Timer=true;
    Testinv.Debug=false;
	printf("Process Bands\n");
	Testinv.Process_AllBands(Name_Cube_In,Name_Beams,Name_Weights,Name_Struc,Name_Out);    
  
    gettimeofday(&end, (struct timezone *) NULL);
	timersub(&end,&start,&diff);
    printf("Total Time=%f\n",(double) diff.tv_sec+(double)diff.tv_usec/1000000.);



    exit(0);
}
